require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

Brick = Object:extend()


function Brick.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function Brick:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	self.currentBBox = nil
	
	self.x = 0
	self.y = 0
	
	self.row = 0
	self.column = 0
	
	self.type = 0
	
	self.rotateLeft = false
	
	self.rotStep = 0
	self.rotProgress = 0
	self.rotThresh = 8
	
end

function Brick.setRotation(self, aRot)
	self.rotateLeft = aRot
end

function Brick.putPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.adjustBoundingBox(self)
end

function Brick.setType(self, aType)

	self.type = aType
end

function Brick.activate(self, aRow, aCol)
	
	self.active = true
	
	self.row = aRow
	self.column = aCol
	
	self.currentGph = enmBrickGph[self.type]
	
	self.currentBBox = BoundingBox(enmBrickBoxGph)
	
	self.scoreReward = 10
	
	self.enmWidth  = 5
	self.enmHeight = 3
	
	self.hitPoints = 1
	
	self.rotStep = 1
	
	--deactivate empty bricks right away so empty bricks will not be calculated to take hits
	if self.type == 4 then
		self.deactivate(self)
	end
	
end

function Brick.plantExplosion(self)

	--no explosion here
	
end

function Brick.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function Brick.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function Brick.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				--print("Enemy dies")
					
				if globIsInDemoMode == false then 
					enemyHit2Snd:stop()
					enemyHit2Snd:play()
				end
				
				self.deactivate(self)
			end
		end
	end
	
	if self.active == true then
		--rotate brick of type 2
		
		if self.type == 2 then
	
			self.rotProgress = self.rotProgress + self.rotStep
			
			if self.rotProgress >= self.rotThresh then
				self.rotProgress = 0
				
				if self.rotateLeft == true then
					self.column = self.column - 1
					if self.column < 0 then
						self.column = 8
					end
				else
					self.column = self.column + 1 
					if self.column > 8 then
						self.column = 0
					end
				end
			end
		end
	end
	
end

function Brick.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end

function Brick.adjustBoundingBox(self)

	--1 frame, 1 box, simplified version
	self.currentBBox.doUpdate(self.currentBBox, self.x, self.y)
end







