require "luas.point"

CatmullRomSpline = Object:extend()


function CatmullRomSpline.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function CatmullRomSpline:reset()

	self.calculated = false

	self.refPoints = {}
	self.refPoints[1] = nil
	self.refPoints[2] = nil
	self.refPoints[3] = nil
	self.refPoints[4] = nil
	
	self.points = {}
	self.ptIndexMax = 0 --invalid value
	
	self.stepWidth = 1.0
	self.goodDist = 2.0 --default value for desired distance between two points on the catmull-rom-track
	
end

function CatmullRomSpline.setGoodDist(self, aGD)

	self.goodDist = aGD
end

function CatmullRomSpline.setRefPoints(self, aRefPt1, aRefPt2, aRefPt3, aRefPt4)

	self.refPoints[1] = aRefPt1
	self.refPoints[2] = aRefPt2
	self.refPoints[3] = aRefPt3
	self.refPoints[4] = aRefPt4
end

function CatmullRomSpline.calculate(self)

	--calculate the length between refPoint2 and refPoint3
	local numSubDivisionPts = 1
	local lclP2P3VectorX = self.refPoints[3].x - self.refPoints[2].x
	local lclP2P3VectorY = self.refPoints[3].y - self.refPoints[2].y
	local lclLen = math.sqrt(lclP2P3VectorX * lclP2P3VectorX + lclP2P3VectorY * lclP2P3VectorY)
	local lclDist = lclLen
	
	--determine how many subdivisions are needed until two subsequent points on the resulting catmull-rom spline are roughly the distance determined by goodDist
	while lclDist > self.goodDist do
		numSubDivisionPts = numSubDivisionPts + 1
		lclDist = lclLen/numSubDivisionPts
	end
	
	
	local lclStepWidthLookahead = self.stepWidth/numSubDivisionPts
	
	
	--have a lookahead calculation for the beginning of the spline
	local lclCompLookahead1X = 2*self.refPoints[2].x
	local lclCompLookahead1Y = 2*self.refPoints[2].y
	local lclCompLookahead2X = ((-1)*self.refPoints[1].x + self.refPoints[3].x)*lclStepWidthLookahead
	local lclCompLookahead2Y = ((-1)*self.refPoints[1].y + self.refPoints[3].y)*lclStepWidthLookahead
	local lclCompLookahead3X = (2*self.refPoints[1].x - 5*self.refPoints[2].x + 4* self.refPoints[3].x - self.refPoints[4].x)*lclStepWidthLookahead*lclStepWidthLookahead
	local lclCompLookahead3Y = (2*self.refPoints[1].y - 5*self.refPoints[2].y + 4* self.refPoints[3].y - self.refPoints[4].y)*lclStepWidthLookahead*lclStepWidthLookahead
	local lclCompLookahead4X = ((-1)*self.refPoints[1].x + 3*self.refPoints[2].x - 3*self.refPoints[3].x + self.refPoints[4].x)*lclStepWidthLookahead*lclStepWidthLookahead*lclStepWidthLookahead
	local lclCompLookahead4Y = ((-1)*self.refPoints[1].y + 3*self.refPoints[2].y - 3*self.refPoints[3].y + self.refPoints[4].y)*lclStepWidthLookahead*lclStepWidthLookahead*lclStepWidthLookahead
	
	local lclResultLookaheadX = 0.5*(lclCompLookahead1X + lclCompLookahead2X + lclCompLookahead3X + lclCompLookahead4X)
	local lclResultLookaheadY = 0.5*(lclCompLookahead1Y + lclCompLookahead2Y + lclCompLookahead3Y + lclCompLookahead4Y)
	--calculate the length of the first segment of the spline
	local lclDirLookaheadX = lclResultLookaheadX - self.refPoints[2].x
	local lclDirLookaheadY = lclResultLookaheadY - self.refPoints[2].y
	local lclLenLookahead  = math.sqrt(lclDirLookaheadX * lclDirLookaheadX + lclDirLookaheadY * lclDirLookaheadY)
	
	--repeat calculating the first segment on smaller subdivision until the lenght of the first segment is not longer than the tolerancevalue of goodDist*1.4 anymore
	while lclLenLookahead > self.goodDist*1.4 do
	
		numSubDivisionPts = numSubDivisionPts + 1 --have a smaller subdivision
		
		lclStepWidthLookahead = self.stepWidth/numSubDivisionPts
		
		--recalculate the spline segment
		lclCompLookahead1X = 2*self.refPoints[2].x
		lclCompLookahead1Y = 2*self.refPoints[2].y
		lclCompLookahead2X = ((-1)*self.refPoints[1].x + self.refPoints[3].x)*lclStepWidthLookahead
		lclCompLookahead2Y = ((-1)*self.refPoints[1].y + self.refPoints[3].y)*lclStepWidthLookahead
		lclCompLookahead3X = (2*self.refPoints[1].x - 5*self.refPoints[2].x + 4* self.refPoints[3].x - self.refPoints[4].x)*lclStepWidthLookahead*lclStepWidthLookahead
		lclCompLookahead3Y = (2*self.refPoints[1].y - 5*self.refPoints[2].y + 4* self.refPoints[3].y - self.refPoints[4].y)*lclStepWidthLookahead*lclStepWidthLookahead
		lclCompLookahead4X = ((-1)*self.refPoints[1].x + 3*self.refPoints[2].x - 3*self.refPoints[3].x + self.refPoints[4].x)*lclStepWidthLookahead*lclStepWidthLookahead*lclStepWidthLookahead
		lclCompLookahead4Y = ((-1)*self.refPoints[1].y + 3*self.refPoints[2].y - 3*self.refPoints[3].y + self.refPoints[4].y)*lclStepWidthLookahead*lclStepWidthLookahead*lclStepWidthLookahead
	
		lclResultLookaheadX = 0.5*(lclCompLookahead1X + lclCompLookahead2X + lclCompLookahead3X + lclCompLookahead4X)
		lclResultLookaheadY = 0.5*(lclCompLookahead1Y + lclCompLookahead2Y + lclCompLookahead3Y + lclCompLookahead4Y)
		--calculate the length of the first segment of the spline
		lclDirLookaheadX = lclResultLookaheadX - self.refPoints[2].x
		lclDirLookaheadY = lclResultLookaheadY - self.refPoints[2].y
		lclLenLookahead  = math.sqrt(lclDirLookaheadX * lclDirLookaheadX + lclDirLookaheadY * lclDirLookaheadY)
		
		--print("SPLINE SEGMENT WAS TOO LONG AND WAS ADJUSTED")
	
	end
	
	--now numSubDivisionPts should have the proper value to calculate the result spline
	
	--now determine the stepWidth on the catmull-rom spline to move inside the intervall of [0.0, 1.0]
	self.stepWidth = self.stepWidth/numSubDivisionPts
	
	--calculate the spline 
	local lclStepProgress = 0.0
	
	for i=1,numSubDivisionPts,1 do
	
		lclStepProgress = lclStepProgress + self.stepWidth
		
		--break the formula down to components
		local lclComp1X = 2*self.refPoints[2].x
		local lclComp1Y = 2*self.refPoints[2].y
		local lclComp2X = ((-1)*self.refPoints[1].x + self.refPoints[3].x)*lclStepProgress
		local lclComp2Y = ((-1)*self.refPoints[1].y + self.refPoints[3].y)*lclStepProgress
		local lclComp3X = (2*self.refPoints[1].x - 5*self.refPoints[2].x + 4* self.refPoints[3].x - self.refPoints[4].x)*lclStepProgress*lclStepProgress
		local lclComp3Y = (2*self.refPoints[1].y - 5*self.refPoints[2].y + 4* self.refPoints[3].y - self.refPoints[4].y)*lclStepProgress*lclStepProgress
		local lclComp4X = ((-1)*self.refPoints[1].x + 3*self.refPoints[2].x - 3*self.refPoints[3].x + self.refPoints[4].x)*lclStepProgress*lclStepProgress*lclStepProgress
		local lclComp4Y = ((-1)*self.refPoints[1].y + 3*self.refPoints[2].y - 3*self.refPoints[3].y + self.refPoints[4].y)*lclStepProgress*lclStepProgress*lclStepProgress
		
		local lclResultX = 0.5*(lclComp1X + lclComp2X + lclComp3X + lclComp4X)
		local lclResultY = 0.5*(lclComp1Y + lclComp2Y + lclComp3Y + lclComp4Y)
	
		--add point to catmull-rom track
		self.ptIndexMax = self.ptIndexMax + 1
		self.points[self.ptIndexMax] = Point()
		self.points[self.ptIndexMax].putPosition(self.points[self.ptIndexMax], lclResultX, lclResultY)
		
	end

end


