require "luas.shortcuts"
require "luas.globals"

DemoMode = Object:extend()

function DemoMode.new(self)

	self.leaveStateReady  = false
	self.leaveStateWithDevice = GLOB_INPUT_WAITING
	
	--need those to determine which input device was used to exit demo.
	--it will be the device that is used to control the 1st player.
	self.quitWithKeyboardFirePress = false
	self.quitWithGamepadFirePress  = false
	self.quitWithGamepad2FirePress = false
	self.quitWithGamepad3FirePress = false
	self.quitWithMouseFirePress    = false
	self.blockMouseOnWindowbarY    = 0	--must not start game or join player when window gains focus and
	                                    --mouse gets click on windowbar instead of playfield
	
	self.leaveStateDemoCountdown      = 0 --SEPARATE FROM GAMEOVER FLAG: start with gameover, block return to demo for short time
	self.LEAVE_STATE_LEAVE_DEMO_DELAY = 40--SEPARATE FROM GAMEOVER FLAG: min time before return to demo
	
	self.levelBackGph   = backGroundGph
	self.levelGroundGph = foreGroundGph
	self.levelBackFlickerGph   = backGroundFlickerGph
	self.levelGroundFlickerGph = foreGroundFlickerGph
	
	self.bkg_x = 0
	self.bkg_y = 0
	
	self.levelLoaded  = false --start of not having loaded the level
	self.levelCleared = false --start of not having not cleared the level
	
	self.levelClearedCountDown = 0
	
	self.playerTookHit = {} 
	self.playerTookHit[1] = false--can't access maxplayers from globals
	--self.playerTookHit[2] = false --only 1 player
	--self.playerTookHit[3] = false--with this construct can let multiple players get hit and die simultaneously
	self.flickerCountDown       = 0
	self.flickerCountDownMax    = 10
	
	-- with this, force smallUpdates only during playerDying Phase
	self.doUpdatesFlag = true
	
	--don't let player join when mouse-event on window-bar instead of playfield is worked through
	self.blockMouseOnWindowbarY = 0
	
	self.debug = 0
	
	
	--demo only: variables for titlescreen handling, showing logos ect
	
	self.titleFrame = 1 
	self.titleScreenCountdownMax = 360
	self.titleScreenCountdown    = self.titleScreenCountdownMax
	
	self.demoDeathFontCount = 0
	self.demoDeathFontCountMax = 4
	
	self.demoGroundRecolorCount = 0
	self.demoGroundRecolorCountMax = 15
	
end


function DemoMode:update(dt)
	--do not depend on deltatime dt here, use doUpdate() instead
end

--replace update
function DemoMode:doUpdate()																	--Task AAQ (part 2) update demo mode

	if self.leaveStateDemoCountdown < self.LEAVE_STATE_LEAVE_DEMO_DELAY then
		--
		self.leaveStateDemoCountdown = self.leaveStateDemoCountdown + 1
	else
		self:keyProcess()
	end
	
	
	
	--if self.isPaused == false then--no pause in demo
																--Task AAM (part 2) update game mode

		--##########################################################################
		--# fill level according to current global level number (NO-UPDATE-FUNCTION)
		--##########################################################################
		
		if self.doUpdatesFlag == true then
			if self.levelLoaded == false then  --no-update scenario
		
				--adjust bulletspeeds, enemyspeeds ect to get more harsh the higher the level is
				globLevelSettings.doCustomize(globLevelSettings, globLevel)
		
				self.levelLoaded = true
				globFillLevel()
			end
		end
		
	
		--#######################################################
		--# do joining of 2nd or 3rd player  (NO-UPDATE-FUNCTION)  --don't have this in demomode
		--#######################################################
	
	
		--#######################
		--# update player       #
		--#######################
		
		
		for i = 1,globMaxPlayers,1 do
	
			if self.doUpdatesFlag == true then

				--FULL UPDATE
			
				if self.debug == 0 then
					self.debug = self.debug + 1
					print("    +++ DOING   FIRST   NORMAL  UPDATE   IN   A   ROW +++  ")
				end
			
				globPlShip[i].doUpdate(globPlShip[i]) --different players need different shipcolors
			end
		end
	
	
		--#######################
		--# update player bullets
		--#######################
		
		for i=1,globMaxPlayers,1 do 
			for j=1,globLeadPlBulletIndexMax,1 do
		
				if self.doUpdatesFlag == true then

					--FULL UPDATE
					globLeadPlBullets[i][j].doUpdate(globLeadPlBullets[i][j])
				end
			end
		end
	
	
		--###############################################################
		--# do playerbullet-enemy collision handling (NO-UPDATE-FUNCTION)
		--###############################################################
		
		
		if self.doUpdatesFlag == true then

			self.playerBulletEnmCollisionHandling(self) --no-update function
		end
	
	
		--#############################
		--# decide for enemies shooting
		--#############################
		
		if self.doUpdatesFlag == true then
			self.prepareEnemyAShooting_Lv1(self)
		end
		
		if self.doUpdatesFlag == true then
			self.prepareEnemyBShooting_Lv1(self)
		end
		
		if self.doUpdatesFlag == true then
			self.prepareEnemyCShooting_Lv1(self)
		end
		
		if self.doUpdatesFlag == true then
			self.prepareEnemyDShooting_Lv1(self)
		end
	
	
		--#######################
		--# update enemies 
		--#######################
	
		for i = 1,globMaxEnemyA,1 do
			if globEnemyA[i] ~= nil then
		
				if self.doUpdatesFlag == true then
			
					--FULL UPDATE
					globEnemyA[i].doUpdate(globEnemyA[i])
				end
			end
		end
		
		for i = 1,globMaxEnemyB,1 do
			if globEnemyB[i] ~= nil then
		
				if self.doUpdatesFlag == true then
			
					--FULL UPDATE
					globEnemyB[i].doUpdate(globEnemyB[i])
				end
			end
		end
		
		for i = 1,globMaxEnemyC,1 do
			if globEnemyC[i] ~= nil then
		
				if self.doUpdatesFlag == true then
			
					--FULL UPDATE
					globEnemyC[i].doUpdate(globEnemyC[i])
				end
			end
		end
		
		for i = 1,globMaxEnemyD,1 do
			if globEnemyD[i] ~= nil then
		
				if self.doUpdatesFlag == true then
			
					--FULL UPDATE
					globEnemyD[i].doUpdate(globEnemyD[i])
				end
			end
		end
		
		for i = 1,globMaxEnemyE,1 do
			if globEnemyE[i] ~= nil then
		
				if self.doUpdatesFlag == true then
			
					--FULL UPDATE
					globEnemyE[i].doUpdate(globEnemyE[i])
				end
			end
		end
	
		--#######################
		--# update big enemies 
		--#######################
	
	
		--#######################
		--# decide spawn enemyE
		--#######################
		
		if self.doUpdatesFlag == true then
			self.prepareEnemyESpawn(self) --no-update function
		end	
		
		
		--#######################
		--# decide enemyC diving
		--#######################
		
		if self.doUpdatesFlag == true then
			self.doUpdateNumEnemyCSwirling(self)
			self.prepareEnemyCSwirl(self)
		end


		--#######################
		--# update enemy bullets
		--#######################
		
		for i = 1, globEnmBullets1IndexMax, 1 do
	
			if globEnmBullets1[i] ~= nil then
		
				if self.doUpdatesFlag == true then

					--FULL UPDATE
					globEnmBullets1[i].doUpdate(globEnmBullets1[i])
				end
			end
		end
		
		for i = 1, globEnmBullets2IndexMax, 1 do
	
			if globEnmBullets2[i] ~= nil then
		
				if self.doUpdatesFlag == true then

					--FULL UPDATE
					globEnmBullets2[i].doUpdate(globEnmBullets2[i])
				end
			end
		end
		
		for i = 1, globEnmBullets3IndexMax, 1 do
	
			if globEnmBullets3[i] ~= nil then
		
				if self.doUpdatesFlag == true then

					--FULL UPDATE
					globEnmBullets3[i].doUpdate(globEnmBullets3[i])
				end
			end
		end
	
	
		--##########################
		--# update enemy BIG bullets
		--##########################
	
	
		--#######################
		--# update explosions
		--#######################
		
		for i=1,globExplosionAMax,1 do
			if globExplosionA[i] ~= nil then
		
				if self.doUpdatesFlag == true then

					globExplosionA[i].doUpdate(globExplosionA[i])
				end
		
			end
		end
		
		for i=1,globExplosionBMax,1 do
			if globExplosionB[i] ~= nil then
		
				if self.doUpdatesFlag == true then

					globExplosionB[i].doUpdate(globExplosionB[i])
				end
		
			end
		end
		
		for i=1,globExplosionCMax,1 do
			if globExplosionC[i] ~= nil then
		
				if self.doUpdatesFlag == true then

					globExplosionC[i].doUpdate(globExplosionC[i])
				end
		
			end
		end
	
	
		--#######################
		--# update BIG explosions
		--#######################
	
	
		--##########################
		--# update Player explosions
		--##########################
		
		for i=1,globPlayerExplosionMax,1 do
			if globPlayerExplosions[i] ~= nil then
		
				if self.doUpdatesFlag == true then
				
					globPlayerExplosions[i].doUpdate(globPlayerExplosions[i])
				end
			
			end
		end
	
	
		--#####################################
		--#check for player taking a bullet hit
		--#####################################
		--
		
		if self.doUpdatesFlag == true then
		
			self.enmBulletPlayerCollisionHandling(self)
		end
	
	
		--######################################
		--#check for player colliding with enemy
		--######################################
		--
		
		if self.doUpdatesFlag == true then
		
			self.enemyPlayerCollisionHandling(self)
		end
	
	
		--########################
		--#check for level cleared
		--########################
		
		if self.doUpdatesFlag == true then

			self.levelClearAndFadeoutHandling(self) --no-update function
		end

	
	
		--########################
		--#update mask printer
		--########################
		
		--TODO don't allow update with gamemode on transition to demomode (where some stuff may have been reset already)
	
		--if self.doUpdatesFlag == true then 	--blocking updates for this one may cause irritating behavior 
											--with not actualizing lifes (when there is no correct respawn invulnerability)
			--FULL UPDATE
			local lclDemoFlag = true
			globMaskPrinter.doUpdate(globMaskPrinter, lclDemoFlag) --update as gamemode --TODO in demoMode make difference
		--end
	
	
		--#########################################################
		--# have background flicker on playerdeath only for so long
		--#########################################################
		
		if self.doUpdatesFlag == true then
			self.deathFlickerUseUp(self)
		end
		
		--#############################
		--# update EnemyD death color
		--#############################
		
		if self.doUpdatesFlag == true then
			self.enemyDDeathColorUpdate(self) --in demo mode also use for titlefont recoloring
		end
		
		--###################################################
		--# Demo only - change groundcolor repeatedly
		--###################################################
		
		if self.doUpdatesFlag == true then
			self.demoGroundRecolor(self)
		end
	
		--#########################################
		--#Do Gameover-checks and Gameover handling
		--#########################################
	
		--self.doGameOverCheck(self) --no gameover in demo mode
		
	--end --no pause in demo
	
	
	--self.checkPauseUnPause(self) --no pause in demo
	
	
	--no gameover in demomode, keyprocess is called above
	--
	--if self.gameOverFlag == true then --
		--
	--	if self.leaveStateGameOverCountdown < self.LEAVE_STATE_GAME_OVER_DELAY then
	--		self.leaveStateGameOverCountdown = self.leaveStateGameOverCountdown + 1
	--	else
			--##################################################################################
			--#	Read and handle Controls that need to be processed in Gamestate (for State exit)
			--##################################################################################
			--
	--		self:keyProcess()
	--	end
		--
	--end
	
	
	
	
	--if self.doUpdatesFlag == false then 
										  
	--	self.playerDeathHandling(self)
		
	--end
	
	--#########################################################
	--# DEMO only: update titlescreen, logos, press fire prompt
	--#########################################################
	
	self.doTitlescreenHandling(self)
	
	
end

function DemoMode:keyProcess()
	--
	--
	self.quitWithKeyboardFirePress = lkeyBrd("z") or lkeyBrd("y")
	self.quitWithMouseFirePress    = lMouseDown("l")
	--
	local lclOldMousePos = self.blockMouseOnWindowbarY
	self.blockMouseOnWindowbarY = lMouseGetY()
	
	if self.quitWithMouseFirePress == true then
		print("mouse start on coordinate"..self.blockMouseOnWindowbarY)
	end
	
	if self.blockMouseOnWindowbarY <= 0 then --if mousefire outside playfield (happens on windowbar gaining focus), 
		self.quitWithMouseFirePress = false  --then DON'T react to it
	elseif lclOldMousePos <= 0 and self.blockMouseOnWindowbarY > 0 then
		self.quitWithMouseFirePress = false --even when just re-entering window and mouseevent still keeps prending
		                                    --when is was a click on windowbar then DON'T REACT TO THAT FUCKING MOUSEPRESS
		--TODO: still fail, also see gameMode (checkPlayerJoin)
	end
	
	
	if globJoystick then --condition for joystick plugged in, tested in globals
			
		--dleft  = globJoystick:isGamepadDown("dpleft")
		--dright = globJoystick:isGamepadDown("dpright")
		
		--TODO can do for now, but need to find start button
		--Btns 1,2,3 directional (Number for up-Button Missing?)
		--Btns 8, 9 on top
		--Btns 10, 11, 12, 13 Roundbuttons (Fire)
		--other buttons not yet found up until index 25
		
		--test
		--print("hasjoystick")
		
		--if globJoystick:isDown(10) then
		--	print("button 10 down")
		--end

		--down10 = globJoystick:isDown(10) --use all 4 buttons as fire button 
		--down11 = globJoystick:isDown(11) --use all 4 buttons as fire button 
		--down12 = globJoystick:isDown(12) --use all 4 buttons as fire button 
		--down13 = globJoystick:isDown(13) --use all 4 buttons as fire button 
		
		--if down10 == true or down11 == true or down12 == true or down13 == true then
		--	self.quitWithGamepadFirePress = true
		--end
		
		self.quitWithGamepadFirePress = globJoystick:isGamepadDown("a", "b", "x", "y")
		
		
		--if globJoystick:isDown(1) then
		--else
		--end
	else
		--print("No Joystick")
	end
	
	
	if globJoystick2 then --condition for joystick plugged in, tested in globals
		
		self.quitWithGamepad2FirePress = globJoystick2:isGamepadDown("a", "b", "x", "y")

	else
		--print("No Joystick2")
	end
	
	if globJoystick3 then --condition for joystick plugged in, tested in globals
		
		self.quitWithGamepad3FirePress = globJoystick3:isGamepadDown("a", "b", "x", "y")

	else
		--print("No Joystick2")
	end
	
	
	
	--self.quitWithGamepadFirePress  = --TODO Gamepad
	--self.quitWithMouseFirePress    = --TODO mouse
	
	--device to be carried over to game (use for 1st player)
	if self.quitWithKeyboardFirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_KEYBOARD
		print("DemoMode: Keyboard input detected")
		
	elseif self.quitWithGamepadFirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_GAMEPAD
		print("DemoMode: Gamepad input detected")
		
	elseif self.quitWithGamepad2FirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_GAMEPAD2
		print("DemoMode: Gamepad2 input detected")
	
	elseif self.quitWithGamepad3FirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_GAMEPAD3
		print("DemoMode: Gamepad3 input detected")
	
	elseif self.quitWithMouseFirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_MOUSE
		print("DemoMode: Mouse input detected")
		
	end
end


function DemoMode.enemyDDeathColorUpdate(self)

	self.demoDeathFontCount = self.demoDeathFontCount + 1
	
	if self.demoDeathFontCount >= self.demoDeathFontCountMax then --only every 4th frame in demo
	
		--new color
		self.demoDeathFontCount = 0
	
		local lclNewDeathColor = love.math.random(globEnmDDeathColorIndexMax)
	
		while lclNewDeathColor == globEnmDDeathColorIndex do
			lclNewDeathColor = love.math.random(globEnmDDeathColorIndexMax)
		end
	
		globEnmDDeathColorIndex = lclNewDeathColor
		
		--apply new color to titlefonts
		for i = 1,3,1 do
			dat = gameLogoGph[i]:getData()
			dat:mapPixel( makeReColor3 )
			gameLogoGph[i]:refresh()
		end
	end
end


function DemoMode.playerBulletEnmCollisionHandling(self)

	for i=1,globMaxPlayers,1 do 
		for j=1,globLeadPlBulletIndexMax,1 do
		
			--Enemy A
			for k = 1,globMaxEnemyA,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyA[k] ~= nil and globEnemyA[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyA[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyA[k].x + m 
									testY = globEnemyA[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyA[k].takeHit(globEnemyA[k], 1) --prepare 1 Hp damage in enemy
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			--Enemy B
			for k = 1,globMaxEnemyB,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyB[k] ~= nil and globEnemyB[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyB[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyB[k].x + m 
									testY = globEnemyB[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyB[k].takeHit(globEnemyB[k], 1) --prepare 1 Hp damage in enemy
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			--Enemy C
			for k = 1,globMaxEnemyC,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyC[k] ~= nil and globEnemyC[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyC[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyC[k].x + m 
									testY = globEnemyC[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyC[k].takeHit(globEnemyC[k], 1) --prepare 1 Hp damage in enemy
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			--Enemy D
			for k = 1,globMaxEnemyD,1 do
			
				
				if globEnemyD[k] ~= nil and (globEnemyD[k].active == true or globEnemyD[k].semiActive == true)then
					
					--test collisions with bricks
					local lclTestX = globLeadPlBullets[i][j].x + 1
					local lclTestY = globLeadPlBullets[i][j].y
						
					for m = 1,8,1 do
						for n = 1,9,1 do
						
							if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
							
								local lclBlHit = false
								local lclBulletHitTarget = false
								
								if globEnemyD[k].bricks[m][n].active == true then
									lclBlHit = globEnemyD[k].bricks[m][n].currentBBox.InBoxTest(globEnemyD[k].bricks[m][n].currentBBox,lclTestX, lclTestY)
								end
								
								if lclBlHit == true then
							
									lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
									globEnemyD[k].bricks[m][n].takeHit(globEnemyD[k].bricks[m][n], 1) --prepare 1 Hp damage in enemy
								end
								
								if lclBulletHitTarget == true then
									globLeadPlBullets[i][j]:deactivate()
								end
							end	
								
						end
					end
						
					--test collision with core
					
					if globEnemyD[k].core.active == true then --since deactivation of enemyd is delayed relative to destroying the core it could be that
															  --enemyd is active and core is no longer, so if we dont check for core active the following will crash
						
						if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
					
							local lclBlHit = false
							local lclBulletHitTarget = false
						
						
							local imgdat = globEnemyD[k].core.currentGph:getData()
							local fieldW = imgdat:getWidth()
							local fieldH = imgdat:getHeight()
							local testX = 0
							local testY = 0
						
							local r, g, b, a =  0
					
					
							for m = 1,fieldW, 1 do
								for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
									r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
									if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
										testX = globEnemyD[k].core.x + m 
										testY = globEnemyD[k].core.y + n 
										lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
									end
								end
							end
					
					
							if lclBlHit == true then
							
								lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
								globEnemyD[k].core.takeHit(globEnemyD[k].core, 1) --prepare 1 Hp damage in enemy
								
								self.flickerCountDown = self.flickerCountDownMax
							end
						
							if lclBulletHitTarget == true then
								globLeadPlBullets[i][j]:deactivate()
							end
					
						end
					
					end
						
					
				end
				

				
			end
			
			
			
			
			--Enemy E
			for k = 1,globMaxEnemyE,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyE[k] ~= nil and globEnemyE[k].active == true then
						
						
						local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnemyE[k].currentBBox)
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyE[k].takeHit(globEnemyE[k], 1) --prepare 1 Hp damage in enemy
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			
		end
	end
end


function DemoMode.prepareEnemyAShooting_Lv1(self)


	--simple: just 1 enemy shooting at a time, 1 projectile
	local lclEnmCount   = 0
	local lclEnmShootTrigger = false
	for i = 1,globMaxEnemyA,1 do
		if globEnemyA[i] ~= nil and globEnemyA[i].active == true then
			lclEnmCount = lclEnmCount + 1
		end
	end

	local lclShootByRateRnd = love.math.random(levlAShootRateRndMax)
	if lclShootByRateRnd <= levlAShootRateRnd then
	
		--print("shootRate succeed")
		
		local lclDoShootRnd = love.math.random(levlAShootDoRndMax)
		if lclDoShootRnd <= levlAShootDoRnd then
			
			--print("doshoot delegate")
			
			lclEnmShootTrigger = true
		end	
	end
	if lclEnmShootTrigger == true then --shooting was decided
	
		if lclEnmCount > 0 then --otherwise will crash in leveltransitions. 0 can't be randomized.
			
			--calculate index of the candidate for shooting
			local lclCandidateNum = love.math.random(lclEnmCount)
			if lclCandidateNum == 0 then
				lclCandidateNum = 1
			end
			--lclCandidateNum == n is the n'th available enemy but can't assume it is globEnemy[n]
			--that's because there are gaps. need to re-iterare to evaluate the candidateIndex
			local lclCandidateCount = 0
			for i = 1,globMaxEnemyA,1 do
				if globEnemyA[i] ~= nil and globEnemyA[i].active == true then
					
					lclCandidateCount = lclCandidateCount + 1 
					if lclCandidateCount == lclCandidateNum then
						--found the INDEX, enemy of interest is at index i
						local lclCandidateIndex = i --for emphsis
					
						--tell enemy to shoot, let the enemy handle the specifics of the shooting modalities
						globEnemyA[lclCandidateIndex].prepareShoot(globEnemyA[lclCandidateIndex])
					end
				end
			end
			
		end
	
	end
	

end

function DemoMode.prepareEnemyBShooting_Lv1(self)

	--simple: just 1 enemy shooting at a time, 1 projectile
	local lclEnmCount   = 0
	local lclEnmShootTrigger = false
	for i = 1,globMaxEnemyB,1 do
		if globEnemyB[i] ~= nil and globEnemyB[i].active == true then
			lclEnmCount = lclEnmCount + 1
		end
	end

	local lclShootByRateRnd = love.math.random(levlBShootRateRndMax)
	if lclShootByRateRnd <= levlBShootRateRnd then
	
		--print("shootRate succeed")
		
		local lclDoShootRnd = love.math.random(levlBShootDoRndMax)
		if lclDoShootRnd <= levlBShootDoRnd then
			
			--print("doshoot delegate")
			
			lclEnmShootTrigger = true
		end	
	end
	if lclEnmShootTrigger == true then --shooting was decided
	
		if lclEnmCount > 0 then --otherwise will crash in leveltransitions. 0 can't be randomized.
			
			--calculate index of the candidate for shooting
			local lclCandidateNum = love.math.random(lclEnmCount)
			if lclCandidateNum == 0 then
				lclCandidateNum = 1
			end
			--lclCandidateNum == n is the n'th available enemy but can't assume it is globEnemy[n]
			--that's because there are gaps. need to re-iterare to evaluate the candidateIndex
			local lclCandidateCount = 0
			for i = 1,globMaxEnemyB,1 do
				if globEnemyB[i] ~= nil and globEnemyB[i].active == true then
					
					lclCandidateCount = lclCandidateCount + 1 
					if lclCandidateCount == lclCandidateNum then
						--found the INDEX, enemy of interest is at index i
						local lclCandidateIndex = i --for emphsis
					
						--tell enemy to shoot, let the enemy handle the specifics of the shooting modalities
						globEnemyB[lclCandidateIndex].prepareShoot(globEnemyB[lclCandidateIndex])
					end
				end
			end
			
		end
	
	end

end

function DemoMode.prepareEnemyCShooting_Lv1(self)


	--simple: just 1 enemy shooting at a time, 1 projectile
	local lclEnmCount   = 0
	local lclEnmShootTrigger = false
	for i = 1,globMaxEnemyC,1 do
		if globEnemyC[i] ~= nil and globEnemyC[i].active == true then
			lclEnmCount = lclEnmCount + 1
		end
	end

	local lclShootByRateRnd = love.math.random(levlCShootRateRndMax)
	if lclShootByRateRnd <= levlCShootRateRnd then
	
		--print("shootRate succeed")
		
		local lclDoShootRnd = love.math.random(levlCShootDoRndMax)
		if lclDoShootRnd <= levlCShootDoRnd then
			
			--print("doshoot delegate")
			
			lclEnmShootTrigger = true
		end	
	end
	if lclEnmShootTrigger == true then --shooting was decided
	
		if lclEnmCount > 0 then --otherwise will crash in leveltransitions. 0 can't be randomized.
			
			--calculate index of the candidate for shooting
			local lclCandidateNum = love.math.random(lclEnmCount)
			if lclCandidateNum == 0 then
				lclCandidateNum = 1
			end
			--lclCandidateNum == n is the n'th available enemy but can't assume it is globEnemy[n]
			--that's because there are gaps. need to re-iterare to evaluate the candidateIndex
			local lclCandidateCount = 0
			for i = 1,globMaxEnemyC,1 do
				if globEnemyC[i] ~= nil and globEnemyC[i].active == true then
					
					lclCandidateCount = lclCandidateCount + 1 
					if lclCandidateCount == lclCandidateNum then
						--found the INDEX, enemy of interest is at index i
						local lclCandidateIndex = i --for emphsis
					
						--tell enemy to shoot, let the enemy handle the specifics of the shooting modalities
						globEnemyC[lclCandidateIndex].prepareShoot(globEnemyC[lclCandidateIndex])
					end
				end
			end
			
		end
	
	end

end

function DemoMode.prepareEnemyDShooting_Lv1(self)


	--simple: just 1 enemy shooting at a time, 1 projectile
	local lclEnmCount   = 0
	local lclEnmShootTrigger = false
	for i = 1,globMaxEnemyD,1 do
		if globEnemyD[i] ~= nil and globEnemyD[i].active == true then
			lclEnmCount = lclEnmCount + 1
		end
	end

	local lclShootByRateRnd = love.math.random(levlDShootRateRndMax)
	if lclShootByRateRnd <= levlDShootRateRnd then
	
		--print("shootRate succeed")
		
		local lclDoShootRnd = love.math.random(levlDShootDoRndMax)
		if lclDoShootRnd <= levlDShootDoRnd then
			
			--print("doshoot delegate")
			
			lclEnmShootTrigger = true
		end	
	end
	if lclEnmShootTrigger == true then --shooting was decided
	
		if lclEnmCount > 0 then --otherwise will crash in leveltransitions. 0 can't be randomized.
			
			--calculate index of the candidate for shooting
			local lclCandidateNum = love.math.random(lclEnmCount)
			if lclCandidateNum == 0 then
				lclCandidateNum = 1
			end
			--lclCandidateNum == n is the n'th available enemy but can't assume it is globEnemy[n]
			--that's because there are gaps. need to re-iterare to evaluate the candidateIndex
			local lclCandidateCount = 0
			for i = 1,globMaxEnemyD,1 do
				if globEnemyD[i] ~= nil and globEnemyD[i].active == true then
					
					lclCandidateCount = lclCandidateCount + 1 
					if lclCandidateCount == lclCandidateNum then
						--found the INDEX, enemy of interest is at index i
						local lclCandidateIndex = i --for emphsis
					
						--tell enemy to shoot, let the enemy handle the specifics of the shooting modalities
						globEnemyD[lclCandidateIndex].prepareShoot(globEnemyD[lclCandidateIndex])
					end
				end
			end
			
		end
	
	end

end



function DemoMode.decideShootingLv2(self)


end


function DemoMode.prepareEnemyShooting_Lv2(self)


end


function DemoMode.doUpdateNumEnemyCSwirling(self)
	
	--always keep up to date with the number of enemies swirling at a time
	globNumEnemiesSwirling = 0
	
	for i = 1,globMaxEnemyC,1 do
		if globEnemyC[i] ~= nil and globEnemyC[i].active == true then
			if globEnemyC[i].isDiving or globEnemyC[i].isAscending then
				globNumEnemiesSwirling = globNumEnemiesSwirling + 1
			end
		end
	end	
end

function DemoMode.prepareEnemyCSwirl(self)
	
	local lclEnmCount   = 0
	local lclEnmSwirlTrigger = false
	for i = 1,globMaxEnemyC,1 do
		if globEnemyC[i] ~= nil and globEnemyC[i].active == true then
			lclEnmCount = lclEnmCount + 1
		end
	end

	local lclDiveByRateRnd = love.math.random(levlEnmCDiveRateRndMax)
	if lclDiveByRateRnd <= levlEnmCDiveRateRnd then
		
		local lclDoDiveRnd = love.math.random(levlEnmCDiveDoRndMax)
		if lclDoDiveRnd <= levlEnmCDiveDoRnd then
			
			lclEnmSwirlTrigger = true
		end	
	end
	
	if lclEnmSwirlTrigger == true then --shooting was decided
	
		if lclEnmCount > 0 then --otherwise will crash in leveltransitions. 0 can't be randomized.
			
			--calculate index of the candidate for shooting
			local lclCandidateNum = love.math.random(lclEnmCount)
			if lclCandidateNum == 0 then
				lclCandidateNum = 1
			end
			--lclCandidateNum == n is the n'th available enemy but can't assume it is globEnemy[n]
			--that's because there are gaps. need to re-iterare to evaluate the candidateIndex
			local lclCandidateCount = 0
			for i = 1,globMaxEnemyC,1 do
				if globEnemyC[i] ~= nil and globEnemyC[i].active == true then
			
					lclCandidateCount = lclCandidateCount + 1 
					if lclCandidateCount == lclCandidateNum then
						--found the INDEX, enemy of interest is at index i
						local lclCandidateIndex = i --for emphsis
					
						--tell enemy to shoot, let the enemy handle the specifics of the shooting modalities
						
						if globEnemyC[lclCandidateIndex].isRoaming == true then
						
							if globNumEnemiesSwirling < levlEnmCDiveAllowed then
								globEnemyC[lclCandidateIndex].prepareDive(globEnemyC[lclCandidateIndex])
							end
						end
					end
				end
			end
			
		end
	
	end

end


function DemoMode.prepareEnemyESpawn(self)

	if levlDoSpawnEnemyE == true then
		local lclEnemyECount = 0
		local lclEnemyESpawnTrigger = false
		
		for i = 1,globMaxEnemyE,1 do
			if globEnemyE[i] ~= nil and globEnemyE[i].active == true then
				lclEnemyECount = lclEnemyECount + 1
			end
		end	
		
		local lclEnemyESpawnByRateRnd = love.math.random(levlEnemyESpawnRateRndMax)
		if lclEnemyESpawnByRateRnd <= levlEnemyESpawnRateRnd then
		
			local lclDoEnemyESpawnRnd = love.math.random(levlEnemyESpawnDoRndMax)
			if lclDoEnemyESpawnRnd <= levlEnemyESpawnDoRnd then
			
				lclEnemyESpawnTrigger = true
			end	
		end
		
		if lclEnemyESpawnTrigger == true then
			if lclEnemyECount < levlMaxEnemyEAllowed then
			
				local lclEnmESpawnX = love.math.random(192)
				local lclEnmESpawnY = 150
				
				globEnemyE[globEnemyEIndex].putStartPosition(globEnemyE[globEnemyEIndex], lclEnmESpawnX, lclEnmESpawnY)
				globEnemyE[globEnemyEIndex].setMagnitudeX(globEnemyE[globEnemyEIndex], levlEnemyEMagnX, levlEnemyEMagnRndX)
				globEnemyE[globEnemyEIndex].setMagnitudeY(globEnemyE[globEnemyEIndex], levlEnemyEMagnY, levlEnemyEMagnRndY)
				globEnemyE[globEnemyEIndex].setYSlowdown(globEnemyE[globEnemyEIndex], levlEnemyESlowDwnY)
				
				globEnemyE[globEnemyEIndex].activate(globEnemyE[globEnemyEIndex])
				
				globEnemyEIndex = globEnemyEIndex + 1
				if globEnemyEIndex > globMaxEnemyE then
					globEnemyEIndex = 1
				end
				
			
			end
		end
		
		
	end
end


function DemoMode.levelClearAndFadeoutHandling(self)
	
	self.levelCleared = true
	--remaining EnemyA's
	for i = 1,globMaxEnemyA,1 do
		if globEnemyA[i] ~= nil and globEnemyA[i].active == true then
			self.levelCleared = false
			--debugBotsActive = debugBotsActive + 1
		end
	end	
	--remaining EnemyB's
	for i = 1,globMaxEnemyB,1 do
		if globEnemyB[i] ~= nil and globEnemyB[i].active == true then
			self.levelCleared = false
			--debugBotsActive = debugBotsActive + 1
		end
	end	
	--remaining EnemyC's
	for i = 1,globMaxEnemyC,1 do
		if globEnemyC[i] ~= nil and globEnemyC[i].active == true then
			self.levelCleared = false
			--debugBotsActive = debugBotsActive + 1
		end
	end	
	--remaining EnemyD's
	for i = 1,globMaxEnemyD,1 do
		if globEnemyD[i] ~= nil and globEnemyD[i].active == true then
			self.levelCleared = false
			--debugBotsActive = debugBotsActive + 1
		end
		--
		if globEnemyD[i] ~= nil and globEnemyD[i].semiActive == true then --need to test for semiactive as well or else will have too short leveltransition
			self.levelCleared = false
		end
	end	
	--enemyE not relevant here
	
	if self.levelClearedCountDown == 0 and self.levelCleared == true then
		self.levelClearedCountDown = 100
	end
	if self.levelClearedCountDown > 1 then 
		self.levelClearedCountDown = self.levelClearedCountDown - 1
		
		if self.levelClearedCountDown == 1 then
		
			--Trigger next level
			--globSubLevel = globSubLevel + 1 --Don't leave level in demomode
			
			--if globSubLevel > 4 then
			
			--don't leave overlevel and don't randomize levels in demo mode
			--	globSubLevel = 1
			--	globLevel = globLevel + 1
				
			--	if globLevel > 4 then
					
			--		local lclLevelSelect = love.math.random(750)
				
			--		if lclLevelSelect <= 250 then
			--			globLevel = 5
			--		elseif lclLevelSelect > 500 then
			--			globLevel = 7
			--		else
			--			globLevel = 6
			--		end
					
			--	end
			--end
			
				--adjust bulletspeeds, enemyspeeds ect to get more harsh the higher the level is
				--globLevelSettings.doCustomize(globLevelSettings, globLevel), no, do it where the level loads so level 1 can be included
			
			self.levelCleared = false
			self.levelLoaded  = false
			self.levelClearedCountDown = 0
		
		
		end
	end
	
	
end



function DemoMode.enmBulletPlayerCollisionHandling(self)

	for i=1,globMaxPlayers,1 do 
		
		--Bullet1
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			for j= 1,globEnmBullets1IndexMax,1 do
			
				if globEnmBullets1[j] ~= nil and globEnmBullets1[j].type ~= ENMBULLET_INACTIVE then
					
					
					--local lclTookHit = globEnmBullets[j].currentBBox:CollisionTest(globPlShip[i].currentBBox)
					
					local lclTookHit = false
					
					
					local imgdat = globEnmBullets1[j].currentGph:getData()
					local fieldW = imgdat:getWidth()
					local fieldH = imgdat:getHeight()
					local testX = 0
					local testY = 0
						
					local r, g, b, a =  0
						
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
							if a ~= 0 and lclTookHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX = globEnmBullets1[j].x + m 
								testY = globEnmBullets1[j].y + n 
								
								lclTookHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX, testY)
								
							end
						end
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						globEnmBullets1[j].deactivate(globEnmBullets1[j])
						
					end
					
				end
		
			end
			
		end
		
		
		--Bullet2
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			for j= 1,globEnmBullets2IndexMax,1 do
			
				if globEnmBullets2[j] ~= nil and globEnmBullets2[j].type ~= ENMBULLET_INACTIVE then
					
					
					--local lclTookHit = globEnmBullets[j].currentBBox:CollisionTest(globPlShip[i].currentBBox)
					
					local lclTookHit = false
					
					
					local imgdat = globEnmBullets2[j].currentGph:getData()
					local fieldW = imgdat:getWidth()
					local fieldH = imgdat:getHeight()
					local testX = 0
					local testY = 0
						
					local r, g, b, a =  0
						
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
							if a ~= 0 and lclTookHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX = globEnmBullets2[j].x + m 
								testY = globEnmBullets2[j].y + n 
								
								lclTookHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX, testY)
								
							end
						end
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						globEnmBullets2[j].deactivate(globEnmBullets2[j])
						
					end
					
				end
		
			end
			
		end
		
		
		
		--Bullet3
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			for j= 1,globEnmBullets3IndexMax,1 do
			
				if globEnmBullets3[j] ~= nil and globEnmBullets3[j].type ~= ENMBULLET_INACTIVE then
					
					
					--local lclTookHit = globEnmBullets[j].currentBBox:CollisionTest(globPlShip[i].currentBBox)
					
					local lclTookHit = false
					
					
					local imgdat = globEnmBullets3[j].currentGph:getData()
					local fieldW = imgdat:getWidth()
					local fieldH = imgdat:getHeight()
					local testX = 0
					local testY = 0
						
					local r, g, b, a =  0
						
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
							if a ~= 0 and lclTookHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX = globEnmBullets3[j].x + m 
								testY = globEnmBullets3[j].y + n 
								
								lclTookHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX, testY)
								
							end
						end
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						globEnmBullets3[j].deactivate(globEnmBullets3[j])
						
					end
					
				end
		
			end
			
		end
		
		
		
	end
	
	--initialize player dying and flicker effect for dying
	for i = 1,globMaxPlayers,1 do
		if self.playerTookHit[i] == true then
		
			self.flickerCountDown = self.flickerCountDownMax --used for flicker only
			
			self.playerTookHit[i] = false
					
			globPlShip[i].putToDeath(globPlShip[i])
		end
	end

end



function DemoMode.enemyPlayerCollisionHandling(self)

	for i=1,globMaxPlayers,1 do 
		
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			for j = 1,globMaxEnemyC,1 do
				
				if globEnemyC[j] ~= nil and globEnemyC[j].active == true then
				
				
					local lclTookHit = false
					
					
					local imgdat = globEnemyC[j].currentGph:getData()
					local fieldW = imgdat:getWidth()
					local fieldH = imgdat:getHeight()
					local testX = 0
					local testY = 0
						
					local r, g, b, a =  0
				
				
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
							if a ~= 0 and lclTookHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX = globEnemyC[j].x + m 
								testY = globEnemyC[j].y + n 
								
								lclTookHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX, testY)
								
							end
						end
					end
				
				
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						--globPlShip[i].deactivate(globPlShip[i])	--no longer
						
					end
				
				
				
				end
				
			end	
				
		end
	end
	
	
	for i=1,globMaxPlayers,1 do 
		
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			for j = 1,globMaxEnemyE,1 do
				
				if globEnemyE[j] ~= nil and globEnemyE[j].active == true then
				
				
					local lclTookHit = false
					
					lclTookHit = globPlShip[i].currentBBox:CollisionTest(globEnemyE[j].currentBBox)
				
				
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						--globPlShip[i].deactivate(globPlShip[i])	--no longer
						
					end
				
				
				
				end
				
			end	
				
		end
	end
	
	
	
	--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnemyE[k].currentBBox)
	
	--initialize player dying and flicker effect for dying
	for i = 1,globMaxPlayers,1 do
		if self.playerTookHit[i] == true then
		
			self.flickerCountDown = self.flickerCountDownMax --used for flicker only
			
			self.playerTookHit[i] = false
			
			globPlShip[i].putToDeath(globPlShip[i])
		end
	end
	
end



function DemoMode.deathFlickerUseUp(self)

	if self.flickerCountDown > 0 then --bright background is drawn as long as it is greater 0 to have a flicker effect when player dies
	
		self.flickerCountDown = self.flickerCountDown - 1
	end

end


function DemoMode.doTitlescreenHandling(self)

	self.titleScreenCountdown = self.titleScreenCountdown - 1
	
	if self.titleScreenCountdown > 20 then
		self.titleFrame = 1
	elseif self.titleScreenCountdown <= 20 and self.titleScreenCountdown > 10 then
		self.titleFrame = 3
	else
		self.titleFrame = 2
		
		if self.titleScreenCountdown == 0 then
			self.titleScreenCountdown = 20
		end
	end
end

function DemoMode.demoGroundRecolor(self)

	self.demoGroundRecolorCount = self.demoGroundRecolorCount + 1
	
	if self.demoGroundRecolorCount >= self.demoGroundRecolorCountMax then
		self.demoGroundRecolorCount = 0
		
		--new random groundcolor
		local lclNewGroundColorIndex = love.math.random(globGroundColorIndexMax)
			
		while lclNewGroundColorIndex == globGroundColorIndex do
			lclNewGroundColorIndex = love.math.random(globGroundColorIndexMax)
		end
		globGroundColorIndex = lclNewGroundColorIndex
		
		--apply color to ground
		dat = self.levelGroundGph:getData()
		dat:mapPixel( makeReColor )
		self.levelGroundGph:refresh()
	end
end


function DemoMode:draw()																			--Task AAT (part 3) draw canvas demo mode

	if self.flickerCountDown == 0 then
		lg.draw( self.levelBackGph, self.bkg_x, self.bkg_y, 0, 1, 1)
	else
		lg.draw( self.levelBackFlickerGph, self.bkg_x, self.bkg_y, 0, 1, 1)
	end
	
	if self.flickerCountDown == 0 then
		lg.draw( self.levelGroundGph, self.bkg_x, self.bkg_y, 0, 1, 1) --debug, otherwise below to omit other objects
	else
		lg.draw( self.levelGroundFlickerGph, self.bkg_x, self.bkg_y, 0, 1, 1)
	end
	
	--#######################
	--# draw player 
	--#######################
	
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil then
			globPlShip[i].draw(globPlShip[i])
		end
	end
	
	
	--#######################
	--# draw enemy bullets
	--#######################
	
	for i = 1, globEnmBullets1IndexMax, 1 do
	
		if globEnmBullets1[i] ~= nil then
			globEnmBullets1[i].draw(globEnmBullets1[i])
		end
	end
	
	for i = 1, globEnmBullets2IndexMax, 1 do
	
		if globEnmBullets2[i] ~= nil then
			globEnmBullets2[i].draw(globEnmBullets2[i])
		end
	end
	
	for i = 1, globEnmBullets3IndexMax, 1 do
	
		if globEnmBullets3[i] ~= nil then
			globEnmBullets3[i].draw(globEnmBullets3[i])
		end
	end
	
	
	--#######################
	--# draw enemies 
	--#######################
	
	for i = 1,globMaxEnemyA,1 do
		if globEnemyA[i] ~= nil then
			globEnemyA[i].draw(globEnemyA[i])
		end
	end
	
	for i = 1,globMaxEnemyB,1 do
		if globEnemyB[i] ~= nil then
			globEnemyB[i].draw(globEnemyB[i])
		end
	end
	
	for i = 1,globMaxEnemyC,1 do
		if globEnemyC[i] ~= nil then
			globEnemyC[i].draw(globEnemyC[i])
		end
	end
	
	for i = 1,globMaxEnemyD,1 do
		if globEnemyD[i] ~= nil then
			globEnemyD[i].draw(globEnemyD[i])
		end
	end
	
	--need to draw enemyE in front of maskprinter so it's below
	--for i = 1,globMaxEnemyE,1 do
	--	if globEnemyE[i] ~= nil then
	--		globEnemyE[i].draw(globEnemyE[i])
	--	end
	--end

	
	--#######################
	--# draw player bullets
	--#######################
	
	for i=1,globMaxPlayers,1 do 
		
		for j=1,globLeadPlBulletIndexMax,1 do --globLeadPlBulletIndexMax
			globLeadPlBullets[i][j].draw(globLeadPlBullets[i][j])
		end
	end

	
	--#######################
	--# draw explosions
	--#######################
	
	for i=1,globExplosionAMax,1 do
	
		if globExplosionA[i] ~= nil then
			globExplosionA[i].draw(globExplosionA[i])
		end
	end
	
	for i=1,globExplosionBMax,1 do
	
		if globExplosionB[i] ~= nil then
			globExplosionB[i].draw(globExplosionB[i])
		end
	end
	
	for i=1,globExplosionCMax,1 do
	
		if globExplosionC[i] ~= nil then
			globExplosionC[i].draw(globExplosionC[i])
		end
	end
	
	
	--#################################
	--# draw player explosion
	--#################################
	
	for i=1,globPlayerExplosionMax,1 do
		if globPlayerExplosions[i] ~= nil then
			globPlayerExplosions[i].draw(globPlayerExplosions[i])
		end
	end
	
	--################################
	--# draw Maskprint/Statusbar
	--################################
	
	local LCL_DEMOFLAG = false -- print for NOT demoMode
	globMaskPrinter.draw(globMaskPrinter, LCL_DEMOFLAG) 
	
	
	--#######################################################
	--# draw Enemy E (need to draw in front of maskprinter)
	--#######################################################
	
	for i = 1,globMaxEnemyE,1 do
		if globEnemyE[i] ~= nil then
			globEnemyE[i].draw(globEnemyE[i])
		end
	end
	
	
	--################################
	--# draw Gameover
	--################################
	
	--if self.gameOverFlag == true then
	
	--	lg.draw( gameOverOverlayGph, 0, 0, 0, 1, 1) --no gameover in demomode
	--end
	
	
	--if self.isPaused == true then
	
	--	lg.draw( pauseGph, 0, 0, 0, 1, 1) --no pause in demomode
	--end
	

	--################################
	--# draw Game Logos
	--################################
	lg.draw( gameLogoGph[self.titleFrame], 0, 0, 0, 1, 1)
	
	
	
end

