require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"
require "luas.point"
require "luas.catmullRomSpline"


--HOT TODOS extended loader (enemy bounding box) -> DONE
--          globals: enemy container structure --
--          game + demo mode loading level (?) + enemy from level pictures + activate in container

EnemyB = Object:extend()


function EnemyB.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyB:reset()

	self.active = false	
	
	self.hitPoints = 1
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	--self.currentBBox = nil --no boundingbox for now
	
	
	self.x = 0
	self.y = 0
	
	--relevant for roaming
	self.startPosX = 0 
	self.startPosY = 0
	--

	self.roamAnmIndex    = 0 --invalid values
	self.roamAnmIndexMax = 4
	self.roamAnmStep     = 1
	self.roamAnmThresh   = 6
	
	--####################
	--# Roaming          #
	--####################
	
	--bounds for roaming, put below
	self.roamBoundsLeft  = 0 
	self.roamBoundsRight = 181
	self.roamBoundsTop   = 0
	self.roamBoundsBtm   = 93
	
	self.catmullRomPts = {}
	for i = 1,4,1 do
		self.catmullRomPts[i] = nil
	end
	
	self.splineIndex = 0 --invalid value
	self.currentSpline = nil
	self.goodSplineDist= 0.625 --default
	
	self.plantShotFlag = false
	
end


function EnemyB.putStartPosition(self, aPx, aPy)
	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx --for matrix-type movement and repeatedly recalculating self.x 
	self.startPosY = aPy
	
end


function EnemyB.activate(self, aHp)--aPutOnMatrix, aLvNum) --let bot know speed ect by level
									--aBasicSpeedComp, aRandomSpeedComp, aPutOnMatrix)
									
	--active
	self.active = true	
	
	self.roamAnmIndex    = love.math.random(self.roamAnmIndexMax) --invalid values
	self.roamAnmStep     = 1
	
	self.hitPoints = aHp
	
	self.currentGph = enemyBGph[self.hitPoints][self.roamAnmIndex]
	
	--self.currentBBox = BoundingBox(enmFlyBoxGph) --not used for now

	if self.hitPoints == 1 then
		self.scoreReward = 40
	elseif self.hitPoints == 2 then
		self.scoreReward = 70
	end
	
	self.enmWidth  = 19
	self.enmHeight = 7
	
	--start with valid catmullrom spline for roaming
	self.catmullRomPts[1] = Point()
	self.catmullRomPts[2] = Point()
	self.catmullRomPts[3] = Point()
	self.catmullRomPts[4] = Point()
	--
	self.catmullRomPts[1].putPosition(self.catmullRomPts[1], love.math.random(self.roamBoundsRight), love.math.random(self.roamBoundsBtm))
	self.catmullRomPts[2].putPosition(self.catmullRomPts[2], self.x, self.y)
	self.catmullRomPts[3].putPosition(self.catmullRomPts[3], love.math.random(self.roamBoundsRight), love.math.random(self.roamBoundsBtm))
	self.catmullRomPts[4].putPosition(self.catmullRomPts[4], love.math.random(self.roamBoundsRight), love.math.random(self.roamBoundsBtm))
	
	self.splineIndex = 1 --set to refer to first point in spline
	self.currentSpline =  CatmullRomSpline()
	self.currentSpline.setGoodDist(self.currentSpline, self.goodSplineDist)
	self.currentSpline.setRefPoints(self.currentSpline, self.catmullRomPts[1], self.catmullRomPts[2], self.catmullRomPts[3], self.catmullRomPts[4])
	self.currentSpline.calculate(self.currentSpline)
	
	--self.adjustBoundingBox(self) 
end


function EnemyB.plantExplosion(self)

	globExplosionB[globExplosionBIndex].activate(globExplosionB[globExplosionBIndex], self.currentGph, self.x, self.y)
	
	globExplosionBIndex = globExplosionBIndex + 1
	if globExplosionBIndex > globExplosionBMax then
		globExplosionBIndex = 1
	end
end


function EnemyB.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyB.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyB.doUpdate(self)


	if self.active == true then
	
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				--give score
				--globScore = globScore + self.scoreReward
				--
				--nope: increasing score must do more than that
				--Therefore
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies")
					
				if globIsInDemoMode == false then 
					enemyDie2Snd:stop()
					enemyDie2Snd:play()
				end
				
				self.deactivate(self)
				
			else
			
				if globIsInDemoMode == false then 
					enemyHit2Snd:stop()
					enemyHit2Snd:play()
				end
			end
		end
		
	end
		
	--after moving adjust boundingbox
	if self.active == true then --may be deactivated by now
		
		self.doAnimation(self)
		self.executeRoamMove(self)
		--self.adjustBoundingBox(self) 
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end

end

function EnemyB.doAnimation(self)

	--#############################
	--#   Roaming Animation
	--#############################
	
	self.roamAnmStep = self.roamAnmStep + 1
	
	if self.roamAnmStep >= self.roamAnmThresh then
	
		self.roamAnmStep = 1
	
		self.roamAnmIndex = self.roamAnmIndex + 1
		
		if self.roamAnmIndex > self.roamAnmIndexMax then
			self.roamAnmIndex = 1
			
		end
		
		self.currentGph = enemyBGph[self.hitPoints][self.roamAnmIndex]
	end
	
end
	

function EnemyB.executeRoamMove(self)
	
	
	--#############################
	--#   Roaming Movement
	--#############################
	
	if self.splineIndex > self.currentSpline.ptIndexMax then --criteria for needing new spline
	
		--make it so that the new spline is like appended to the end of the old spline
		self.splineIndex = 1
		self.catmullRomPts[1] = self.catmullRomPts[2]
		self.catmullRomPts[2] = self.catmullRomPts[3]
		self.catmullRomPts[3] = self.catmullRomPts[4]
		self.catmullRomPts[4] = Point()
		self.catmullRomPts[4].putPosition(self.catmullRomPts[4], love.math.random(self.roamBoundsRight), love.math.random(self.roamBoundsBtm))
		
		self.currentSpline =  CatmullRomSpline()
		self.currentSpline.setGoodDist(self.currentSpline, self.goodSplineDist)
		self.currentSpline.setRefPoints(self.currentSpline, self.catmullRomPts[1], self.catmullRomPts[2], self.catmullRomPts[3], self.catmullRomPts[4])
		self.currentSpline.calculate(self.currentSpline)
	
	end
	
	--now for the actual movement	
	self.x = self.currentSpline.points[self.splineIndex].x
	self.y = self.currentSpline.points[self.splineIndex].y
	
	self.splineIndex = self.splineIndex + 1 --prepare splineindex for next movement
		

end


function EnemyB.plantShot(self)


	--if self.active == true then --active was tested outside in this game
	
		local lclLaunchPX = self.x + 2 + love.math.random(15)
		local lclLaunchPY = self.y + 4
		
		local lclPosFactorX = 100 - self.y --make speed dependant on the hight of the enemy on the playfield
		lclPosFactorX = lclPosFactorX/100
		lclPosFactorX = lclPosFactorX*levlBEnmBulSpdPosFactorX
		
		local lclPosFactorY = 100 - self.y --make speed dependant on the hight of the enemy on the playfield
		lclPosFactorY = lclPosFactorY/100
		lclPosFactorY = lclPosFactorY*levlBEnmBulSpdPosFactorY
	
		local lclXSpd = levlBEnmBulSpdBasicX + love.math.random()*levlBEnmBulSpdRndX + love.math.random()*lclPosFactorX
		
		local lclLeftShotRnd = love.math.random(100)
		if lclLeftShotRnd <= 50 then
			lclXSpd = lclXSpd*(-1)
		end
		
		local lclStraightShotRnd = love.math.random(100)
		
		if lclStraightShotRnd <= levlBEnmBulStraightChance then
			lclXSpd = 0.0
		end 

		globEnmBullets2[globEnmBullets2Index].activate(globEnmBullets2[globEnmBullets2Index], lclLaunchPX, lclLaunchPY, lclXSpd, (levlBEnmBulSpdBasicY + love.math.random()*levlBEnmBulSpdRndY + love.math.random()*lclPosFactorY))
		
		globEnmBullets2Index = globEnmBullets2Index + 1
		if globEnmBullets2Index >= globEnmBullets2IndexMax then
			globEnmBullets2Index = 1
		end
		
		if globIsInDemoMode == false then
			shotSnd:stop()
			shotSnd:play()
		end
	--end


end


function EnemyB.prepareShoot(self)
	print("Bot Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end


function EnemyB.draw(self)

	if self.active == true then	
		--take graphics directly from extendedloader for this one
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
		--lg.draw(self.currentGph, self.catmullRomPts[4].x, self.catmullRomPts[4].y, 0, 1, 1)
		
		--debug
		--self.currentBBox.draw(self.currentBBox)
	end
end

function EnemyB.adjustBoundingBox(self)

	--1 frame, 1 box, simplified version
	self.currentBBox.doUpdate(self.currentBBox, self.x, self.y)
end




