require "luas/shortcuts"
require "luas/boundingBox"
require "luas/extendedLoader"

ROAM_INVALID = 0
ROAM_LEFT = 1
ROAM_RIGHT = 2

--HOT TODOS extended loader (enemy bounding box) -> DONE
--          globals: enemy container structure --
--          game + demo mode loading level (?) + enemy from level pictures + activate in container

EnemyC = Object:extend()


function EnemyC.new(self) --, aPosX, aPosY) position only known at activation time
	
	-----------------------------------------------------------------------

	--self.currentBBox = BoundingBox(enmBotBoxGph)--not used yet
	
	-----------------------------------------------------------------------
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyC:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.currentGph  = nil
	
	
	self.enmWidth  = 0
	self.enmHeight = 0
	
	self.boundsLeft  = 0
	self.boundsRight = 179
	self.boundsTop   = 0
	self.boundsBottom   = 108
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.isRoaming = false
	self.isDiving = false
	self.isAscending = false
	
	--####################
	--# Roaming Movement #
	--####################

	--
	self.roamDirection = ROAM_INVALID
	--
	--after latest directionchange, make sure there won't occur another unless a minimum amount of time has passed
	self.roamChangeStep      = 1
	self.roamChangeProgress  = 0 
	self.roamChangeLimit     = 8 
	
	self.roamSpeed      = 0.0   --set on activation
	self.roamDelta		= 0.0	--execute movement 1 pixel in roamdirection when > threshold
	
	self.roamDirChangeTrigger = (-1) --invalid value
	
	
	--used for both diving and ascending
	self.swirlSpeedX = 0.0 --take from levelsettings
	self.swirlMoveProgressX = 0.0
	self.swirlSpeedY = 0.0 --take from levelsettings
	self.swirlMoveProgressY = 0.0
	
	
	self.startDiveFlag = false
	
	
	self.plantShotFlag = false
	
	self.mainColor = 0 --invalid value
	self.subColor  = 0 --invalid value
	
end


function EnemyC.putStartPosition(self, aPx, aPy)
	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx --for matrix-type movement and repeatedly recalculating self.x 
	self.startPosY = aPy
	
	--self.adjustBoundingBox(self) 
end

function EnemyC.activate(self) --let bot know speed ect by level
									--aBasicSpeedComp, aRandomSpeedComp, aPutOnMatrix)

	--active
	self.active = true	
	
	self.hitPoints = 6	
	
	self.scoreReward = 150
	
	
	self.currentGph = enemyCGph --placeholder until recolored
	
	self.enmWidth  = 21
	self.enmHeight = 17
	
	self.isRoaming = true
	

	--randomize a direction to use in case of roaming
	self.randomizeRoamDirection(self)
	
	--randomize initial Roamspeed also.
	self.randomizeNewSpeed(self)
	
	--number of ticks before next direction change for enemy is tested for roaming
	self.randomizeNewDirChangeTime(self)
	
	self.recolor(self)
	
end

function EnemyC.recolor(self)

	local lclNewMainColor = love.math.random(globEnmCMainColorIndexMax)
	
	while lclNewMainColor == self.mainColor do
		lclNewMainColor = love.math.random(globEnmCMainColorIndexMax)
	end
	
	self.mainColor = lclNewMainColor
	
	
	local lclNewSubColor = love.math.random(globEnmCSubColorIndexMax)
	
	while lclNewSubColor == self.subColor do
		lclNewSubColor = love.math.random(globEnmCSubColorIndexMax)
	end
	
	self.subColor = lclNewSubColor
	
	
	--apply new colors
	
	local locImageData = love.image.newImageData( 21, 17)
	locImageData:paste( enemyCGph:getData(), 0, 0, 0, 0, 21, 17 )
	self.currentGph = lg.newImage(locImageData)
	self.currentGph:setFilter("nearest","nearest")
	
	globEnmCMainColorIndex = self.mainColor
	globEnmCSubColorIndex = self.subColor
	
	dat = self.currentGph:getData()
	dat:mapPixel( makeReColor2 )
	self.currentGph:refresh()
	
end

function EnemyC.randomizeRoamDirection(self)

	local lclRandomize       = love.math.random(100)
	
	if lclRandomize <= 50 then
		self.roamDirection = ROAM_LEFT --left
	else
		self.roamDirection = ROAM_RIGHT --right
	end
end

function EnemyC.randomizeNewSpeed(self)

	self.roamSpeed = levlEnmCRoamSpdBasic + love.math.random()*levlEnmCRoamSpdExtra
end

function EnemyC.randomizeNewDirChangeTime(self)

	self.roamDirChangeTrigger = levlEnmCDirChangeTimeBasic + love.math.random(levlEnmCDirChangeTimeExtra)
end

function EnemyC.plantExplosion(self)

	globExplosionC[globExplosionCIndex].activate(globExplosionC[globExplosionCIndex], self.x - 2, self.y - 2, self.mainColor)
	globExplosionCIndex = globExplosionCIndex + 1
	if globExplosionCIndex > globExplosionCMax then
		globExplosionCIndex = 1
	end
end


function EnemyC.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyC.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyC.doUpdate(self)


	if self.active == true then
	
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				
				if globIsInDemoMode == false then 
					enemyDie3Snd:stop()
					enemyDie3Snd:play()
				end
				
				self.deactivate(self)
				
			else
				self.recolor(self)
				
				if globIsInDemoMode == false then
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end

			end
		end
		
	end
	
	if self.active == true then
	
		if self.isRoaming == true then
			self.executeRoamingMove(self) -- if don't wriggle then roam
		end
		
		if self.isDiving == true then
			self.executeDiveMove(self)
		end
		
		if self.isAscending == true then
			self.executeAscendMove(self)
		end
			
		
		
		if self.startDiveFlag == true then
			self.startDiveFlag = false
			self.startDive(self)
		end
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
		
	end

end

function EnemyC.plantShot(self)


	--if self.active == true then --active was tested outside in this game
	
		local lclLaunchPX = self.x + love.math.random(17)
		local lclLaunchPY = self.y + 7
		
		local lclPosFactorX = 100 - self.y --make speed dependant on the hight of the enemy on the playfield
		lclPosFactorX = lclPosFactorX/100
		lclPosFactorX = lclPosFactorX*levlCEnmBulSpdPosFactorX
		
		local lclPosFactorY = 100 - self.y --make speed dependant on the hight of the enemy on the playfield
		lclPosFactorY = lclPosFactorY/100
		lclPosFactorY = lclPosFactorY*levlCEnmBulSpdPosFactorY
	
		local lclXSpd = levlCEnmBulSpdBasicX + love.math.random()*levlCEnmBulSpdRndX + love.math.random()*lclPosFactorX
		
		local lclLeftShotRnd = love.math.random(100)
		if lclLeftShotRnd <= 50 then
			lclXSpd = lclXSpd*(-1)
		end
		
		local lclStraightShotRnd = love.math.random(100)
		
		if lclStraightShotRnd <= levlCEnmBulStraightChance then
			lclXSpd = 0.0
		end 

		globEnmBullets3[globEnmBullets3Index].activate(globEnmBullets3[globEnmBullets3Index], lclLaunchPX, lclLaunchPY, lclXSpd, (levlCEnmBulSpdBasicY + love.math.random()*levlCEnmBulSpdRndY + love.math.random()*lclPosFactorY))
		
		globEnmBullets3Index = globEnmBullets3Index + 1
		if globEnmBullets3Index >= globEnmBullets3IndexMax then
			globEnmBullets3Index = 1
		end
		
		if globIsInDemoMode == false then 
			shotSnd:stop()
			shotSnd:play()
		end
	--end

	
end

function EnemyC.executeRoamingMove(self)

	--#determine roaming
	--###################
	
	self.roamDelta = self.roamDelta + self.roamSpeed
	while self.roamDelta > threshold do
		self.roamDelta = self.roamDelta - 1
		
		if self.roamDirection == ROAM_RIGHT then
			self.x = self.x + 1
		elseif self.roamDirection == ROAM_LEFT then
			self.x = self.x - 1
		end
	end
	
	
	--# check for randomized direction change
	--#######################################
	--
	--Reduce waiting time to next decision
	if self.roamDirChangeTrigger > 0 then
		self.roamDirChangeTrigger = self.roamDirChangeTrigger - 1
	end
	--
	if self.roamDirChangeTrigger == 0 then
	
		--enable next trigger wait time 
		self.randomizeNewDirChangeTime(self)
		
		local lclDirChangeRnd = love.math.random(100)
		
		if lclDirChangeRnd < 70 then --slight preference to keep direction and speed
		
			self.randomizeRoamDirection(self)
			self.randomizeNewSpeed(self)
			
		end
	end
	

	--#watch boundaries (direction change to the opposite side and direct position adjustment to prevent deadlock situations)
	--#################
	if self.x < self.boundsLeft then
	
		self.x = self.boundsLeft
		self.roamDirection = ROAM_RIGHT
	end
	if self.x > self.boundsRight then
	
		self.x = self.boundsRight
		self.roamDirection = ROAM_LEFT
	end


	--self.adjustBoundingBox(self) 
	


end



function EnemyC.executeDiveMove(self)


	--self.swirlSpeedX = 0.0 --take from levelsettings
	--self.swirlMoveProgressX = 0.0
	--self.swirlSpeedY = 0.0 --take from levelsettings
	--self.swirlMoveProgressY = 0.0

	self.swirlMoveProgressX = self.swirlMoveProgressX + self.swirlSpeedX

	self.swirlMoveProgressY = self.swirlMoveProgressY + self.swirlSpeedY
	
	while self.swirlMoveProgressX >= threshold do --for positive x-Movement
		self.swirlMoveProgressX = self.swirlMoveProgressX - threshold
		self.x = self.x + 1
	end
	while self.swirlMoveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.swirlMoveProgressX = self.swirlMoveProgressX + threshold
		self.x = self.x - 1
	end
	
	while self.swirlMoveProgressY >= threshold do --for positive x-Movement
		self.swirlMoveProgressY = self.swirlMoveProgressY - threshold
		self.y = self.y + 1
	end
	--while self.swirlMoveProgressY <= ((-1) * threshold) do --never
	--	self.swirlMoveProgressY = self.swirlMoveProgressY + threshold
	--	self.y = self.y - 1
	--end
	
	if self.x < self.boundsLeft then
		if self.swirlSpeedX < 0 then --only when smaller 0 so that one cannot get stuck in a alternate loop on the edges
			self.swirlSpeedX = self.swirlSpeedX*(-1)
		end
	end 
	if self.x > self.boundsRight then
		if self.swirlSpeedX > 0 then--only when greater 0 so that one cannot get stuck in a alternate loop on the edges
			self.swirlSpeedX = self.swirlSpeedX*(-1)
		end
	end
	if self.y > self.boundsBottom then
		self.startAscend(self)
	end
	
end	

function EnemyC.executeAscendMove(self)

	self.swirlMoveProgressX = self.swirlMoveProgressX + self.swirlSpeedX

	self.swirlMoveProgressY = self.swirlMoveProgressY + self.swirlSpeedY
	
	while self.swirlMoveProgressX >= threshold do --for positive x-Movement
		self.swirlMoveProgressX = self.swirlMoveProgressX - threshold
		self.x = self.x + 1
	end
	while self.swirlMoveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.swirlMoveProgressX = self.swirlMoveProgressX + threshold
		self.x = self.x - 1
	end
	
	--while self.swirlMoveProgressY >= threshold do --never
	--	self.swirlMoveProgressY = self.swirlMoveProgressY - threshold
	--	self.y = self.y + 1
	--end
	while self.swirlMoveProgressY <= ((-1) * threshold) do --never
		self.swirlMoveProgressY = self.swirlMoveProgressY + threshold
		self.y = self.y - 1
	end
	
	if self.x < self.boundsLeft then
		if self.swirlSpeedX < 0 then --only when smaller 0 so that one cannot get stuck in a alternate loop on the edges
			self.swirlSpeedX = self.swirlSpeedX*(-1)
		end
	end 
	if self.x > self.boundsRight then
		if self.swirlSpeedX > 0 then--only when greater 0 so that one cannot get stuck in a alternate loop on the edges
			self.swirlSpeedX = self.swirlSpeedX*(-1)
		end
	end
	if self.y <= self.startPosY then
		self.y = self.startPosY
		self.startRoam(self)
	end

end

function EnemyC.startRoam(self)
	self.isRoaming = true
	self.isDiving = false
	self.isAscending = false
end

function EnemyC.startAscend(self)
	self.isRoaming = false
	self.isDiving = false
	self.isAscending = true
	
	if self.swirlSpeedY > 0 then --revert ydir to move upwards
		self.swirlSpeedY = self.swirlSpeedY*(-1)
	end
end

function EnemyC.startDive(self)

	self.isRoaming = false
	self.isDiving = true
	self.isAscending = false
	
	local lclPosFactor = 100 - self.y --make speed dependant on the hight of the enemy on the playfield
	lclPosFactor = lclPosFactor/100
	
	self.swirlSpeedX = levlEnmCDiveSpdBasicX + love.math.random()*levlEnmCDiveSpdRndX + love.math.random()*levlEnmCDiveSpdPosfactorX*lclPosFactor
	self.swirlSpeedY = levlEnmCDiveSpdBasicY + love.math.random()*levlEnmCDiveSpdRndY + love.math.random()*levlEnmCDiveSpdPosfactorY*lclPosFactor
	
	local lclDirXRnd = love.math.random(100)
	if lclDirXRnd <= 50 then
		self.swirlSpeedX = self.swirlSpeedX*(-1)
	end

end

function EnemyC.prepareDive(self)
	self.startDiveFlag = true
end

function EnemyC.prepareShoot(self)
	print("Bot Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end

function EnemyC.draw(self)

	if self.active then	
		--take graphics directly from extendedloader for this one
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
		
		--debug
		--self.currentBBox.draw(self.currentBBox)
	end
end

function EnemyC.adjustBoundingBox(self)

	--1 frame, 1 box, simplified version
	self.currentBBox.doUpdate(self.currentBBox, self.x, self.y)
end



