require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"
require "luas.brick"
require "luas.core"

EnemyD = Object:extend()


function EnemyD.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyD:reset()

	self.active = false	
	self.semiActive = false
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.bricks = {}
	
	for i = 1,8,1 do
		self.bricks[i] = {}
	end
	
	self.core = nil
	self.corePosX = 15
	self.corePosY = 0
	
	self.boundsLeft = 0
	self.boundsRight = 155
	
	--self.debugMoveX = 0
	
	self.dieCountDown = 0
	
	self.plantShotFlag = false
	
	self.isRoaming = false
	self.isTurning = false
	
	self.speedXPresign = 0
	--
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	--
	self.moveXCount = 0
	self.moveXMax   = 0
	self.moveXBasicCnt = 50
	self.moveXRandomCnt= 500
	--
	self.nextSpeedX = 0.0
	
	self.moveYBasic = 0.4
	self.moveYSlowDown = 0.025
	self.moveYUpsideDown = false
	self.speedY = 0.0
	self.moveProgressY = 0.0
	
end

function EnemyD.putStartPosition(self, aPx, aPy)
	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx --for matrix-type movement and repeatedly recalculating self.x 
	self.startPosY = aPy
	
	--self.adjustBoundingBox(self) --not for this enemy
end

function EnemyD.activate(self)

	print("Enemy D activation")
	
	self.active = true
	
	self.scoreReward = 0 --core gives reward
	
	self.enmWidth  = 45
	self.enmHeight = 24
	
	self.hitPoints = 1
	
	self.buildStructure(self)
	
	--self.debugMoveX = 1 --replace later
	
	self.isRoaming = true
	
	local lclPreSignRnd = love.math.random(100)
	
	if lclPreSignRnd <= 50 then
		self.speedXPresign = (-1)
	else
		self.speedXPresign = 1
	end
	
	self.speedX = 0.7 + love.math.random()*0.5
	--
	self.moveXCount = 0
	self.moveXMax   = self.moveXBasicCnt + love.math.random(self.moveXRandomCnt)
	--
	--self.nextSpeedX = 0.0 --not set yet
	
	self.speedY = self.moveYBasic
	self.moveYUpsideDown = false
	
	--start at random y progress
	local lclDoYLoopsRnd = love.math.random(160)
	
	while lclDoYLoopsRnd > 1 do
		lclDoYLoopsRnd = lclDoYLoopsRnd - 1
		self.doVerticalMovement(self)
	end
end

function EnemyD.buildStructure(self)
	
	self.core = Core()
	self.core.activate(self.core)
	self.core.putPosition(self.core, self.x + self.corePosX, self.y + self.corePosY)
	
	for i = 1,8,1 do 
		for j = 1,9,1 do
			self.bricks[i][j] = Brick()
		end
	end
	
	--1st row
	self.bricks[1][1].setType(self.bricks[1][1], 4)
	self.bricks[1][2].setType(self.bricks[1][2], 4)
	self.bricks[1][3].setType(self.bricks[1][3], 3)
	self.bricks[1][4].setType(self.bricks[1][4], 4)
	self.bricks[1][5].setType(self.bricks[1][5], 4)
	self.bricks[1][6].setType(self.bricks[1][6], 4)
	self.bricks[1][7].setType(self.bricks[1][7], 3)
	self.bricks[1][8].setType(self.bricks[1][8], 4)
	self.bricks[1][9].setType(self.bricks[1][9], 4)
	--2nd row
	self.bricks[2][1].setType(self.bricks[2][1], 4)
	self.bricks[2][2].setType(self.bricks[2][2], 3)
	self.bricks[2][3].setType(self.bricks[2][3], 3)
	self.bricks[2][4].setType(self.bricks[2][4], 4)
	self.bricks[2][5].setType(self.bricks[2][5], 4)
	self.bricks[2][6].setType(self.bricks[2][6], 4)
	self.bricks[2][7].setType(self.bricks[2][7], 3)
	self.bricks[2][8].setType(self.bricks[2][8], 3)
	self.bricks[2][9].setType(self.bricks[2][9], 4)
	--3rd row
	self.bricks[3][1].setType(self.bricks[3][1], 3)
	self.bricks[3][2].setType(self.bricks[3][2], 3)
	self.bricks[3][3].setType(self.bricks[3][3], 3)
	self.bricks[3][4].setType(self.bricks[3][4], 4)
	self.bricks[3][5].setType(self.bricks[3][5], 4)
	self.bricks[3][6].setType(self.bricks[3][6], 4)
	self.bricks[3][7].setType(self.bricks[3][7], 3)
	self.bricks[3][8].setType(self.bricks[3][8], 3)
	self.bricks[3][9].setType(self.bricks[3][9], 3)
	--4th row
	self.bricks[4][1].setType(self.bricks[4][1], 2)
	self.bricks[4][2].setType(self.bricks[4][2], 2)
	self.bricks[4][3].setType(self.bricks[4][3], 2)
	self.bricks[4][4].setType(self.bricks[4][4], 2)
	self.bricks[4][5].setType(self.bricks[4][5], 2)
	self.bricks[4][6].setType(self.bricks[4][6], 2)
	self.bricks[4][7].setType(self.bricks[4][7], 2)
	self.bricks[4][8].setType(self.bricks[4][8], 2)
	self.bricks[4][9].setType(self.bricks[4][9], 2)
	--5th row
	self.bricks[5][1].setType(self.bricks[5][1], 1)
	self.bricks[5][2].setType(self.bricks[5][2], 1)
	self.bricks[5][3].setType(self.bricks[5][3], 1)
	self.bricks[5][4].setType(self.bricks[5][4], 1)
	self.bricks[5][5].setType(self.bricks[5][5], 1)
	self.bricks[5][6].setType(self.bricks[5][6], 1)
	self.bricks[5][7].setType(self.bricks[5][7], 1)
	self.bricks[5][8].setType(self.bricks[5][8], 1)
	self.bricks[5][9].setType(self.bricks[5][9], 1)
	--6th row
	self.bricks[6][1].setType(self.bricks[6][1], 4)
	self.bricks[6][2].setType(self.bricks[6][2], 1)
	self.bricks[6][3].setType(self.bricks[6][3], 1)
	self.bricks[6][4].setType(self.bricks[6][4], 1)
	self.bricks[6][5].setType(self.bricks[6][5], 1)
	self.bricks[6][6].setType(self.bricks[6][6], 1)
	self.bricks[6][7].setType(self.bricks[6][7], 1)
	self.bricks[6][8].setType(self.bricks[6][8], 1)
	self.bricks[6][9].setType(self.bricks[6][9], 4)
	--7th row
	self.bricks[7][1].setType(self.bricks[7][1], 4)
	self.bricks[7][2].setType(self.bricks[7][2], 4)
	self.bricks[7][3].setType(self.bricks[7][3], 1)
	self.bricks[7][4].setType(self.bricks[7][4], 1)
	self.bricks[7][5].setType(self.bricks[7][5], 1)
	self.bricks[7][6].setType(self.bricks[7][6], 1)
	self.bricks[7][7].setType(self.bricks[7][7], 1)
	self.bricks[7][8].setType(self.bricks[7][8], 4)
	self.bricks[7][9].setType(self.bricks[7][9], 4)
	--8th row
	self.bricks[8][1].setType(self.bricks[8][1], 4)
	self.bricks[8][2].setType(self.bricks[8][2], 4)
	self.bricks[8][3].setType(self.bricks[8][3], 4)
	self.bricks[8][4].setType(self.bricks[8][4], 1)
	self.bricks[8][5].setType(self.bricks[8][5], 1)
	self.bricks[8][6].setType(self.bricks[8][6], 1)
	self.bricks[8][7].setType(self.bricks[8][7], 4)
	self.bricks[8][8].setType(self.bricks[8][8], 4)
	self.bricks[8][9].setType(self.bricks[8][9], 4)
	
	local lclRndRot = love.math.random(100)
	local lclRotLeft = false
	if lclRndRot <= 50 then
		lclRotLeft = true
	end
	
	for i = 1,8,1 do 
		for j = 1,9,1 do
			self.bricks[i][j].setRotation(self.bricks[i][j], lclRotLeft)
			self.bricks[i][j].activate(self.bricks[i][j], i-1, j-1)
		end
	end
	
	--need to put brick position after activation when row and column have been set
	for i = 1,8,1 do 
		for j = 1,9,1 do
		
			if self.bricks[i][j].active == true then--empty bricks have been deactivated on activation, so without this line, you get a crash
			
				self.bricks[i][j].putPosition(self.bricks[i][j], self.x + self.bricks[i][j].column*5, self.y + self.bricks[i][j].row*3)
			end
		end
	end
	
	
	
	
end

function EnemyD.plantExplosion(self)

	--no explosion here
	
end

function EnemyD.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyD.doUpdate(self)


	if self.active == true then
	
		--if self.core.hitPoints <= 0 then --hitpoints of entire ship relies on the core
		
		if self.core.active == false then --must not rely on core's hitpoints because on core reset it will be reset to a default of 1, instead check for core inactive
			
			if self.hitPoints > 0 then
				self.hitPoints = self.hitPoints - 1
				
				--following will be executed only once
				self.active = false	
				self.semiActive = true
			end
			
		end
		
	end
	
	if self.active == true then
	
		self.core.doUpdate(self.core)
		
		for i = 1,8,1 do 
			for j = 1,9,1 do
				self.bricks[i][j].doUpdate(self.bricks[i][j])
			end
		end
	
		self.doHorizontalMovement(self)
		self.doVerticalMovement(self)
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
	
	if self.semiActive == true then
		self.doDyingProcess(self)
	end
	
	
end

function EnemyD.plantShot(self)

	
	local lclSpreadRnd = love.math.random(1000)
	local lclDoSpread = false
	
	if lclSpreadRnd <= levlDSpreadshotChance then
		lclDoSpread = true
	end

	
	if lclDoSpread == false then
	
		--normal shooting, shoot from the bricks
		local lclActiveBricksCnt = 0
	
		for i = 1,8,1 do
			for j = 1,9,1 do
				if self.bricks[i][j].active == true then
					lclActiveBricksCnt = lclActiveBricksCnt + 1
				end
			end
		end
	
		if lclActiveBricksCnt > 0 then --can only do normal shot if there is at least one brick left
	
			local lclCandidateNum = love.math.random(lclActiveBricksCnt)
			if lclCandidateNum == 0 then
				lclCandidateNum = 1
			end
		
			local lclCandidateCount = 0
		
			for i = 1,8,1 do
				for j = 1,9,1 do
					if self.bricks[i][j].active == true then
						lclCandidateCount = lclCandidateCount + 1
					
						if lclCandidateCount == lclCandidateNum then
					
					
							local lclLaunchPX = self.bricks[i][j].x + 1
							local lclLaunchPY = self.bricks[i][j].y
		
							local lclPosFactorX = 100 - self.bricks[i][j].y --make speed dependant on the hight of the enemy on the playfield
							lclPosFactorX = lclPosFactorX/100
							lclPosFactorX = lclPosFactorX*levlDEnmBulSpdPosFactorX
		
							local lclPosFactorY = 100 - self.bricks[i][j].y --make speed dependant on the hight of the enemy on the playfield
							lclPosFactorY = lclPosFactorY/100
							lclPosFactorY = lclPosFactorY*levlDEnmBulSpdPosFactorY
	
							local lclXSpd = levlDEnmBulSpdBasicX + love.math.random()*levlDEnmBulSpdRndX + love.math.random()*lclPosFactorX
		
							local lclLeftShotRnd = love.math.random(100)
							if lclLeftShotRnd <= 50 then
								lclXSpd = lclXSpd*(-1)
							end
		
							local lclStraightShotRnd = love.math.random(100)
		
							if lclStraightShotRnd <= levlDEnmBulStraightChance then
								lclXSpd = 0.0
							end 

							globEnmBullets3[globEnmBullets3Index].activate(globEnmBullets3[globEnmBullets3Index], lclLaunchPX, lclLaunchPY, lclXSpd, (levlDEnmBulSpdBasicY + love.math.random()*levlDEnmBulSpdRndY + love.math.random()*lclPosFactorY))
		
							globEnmBullets3Index = globEnmBullets3Index + 1
							if globEnmBullets3Index >= globEnmBullets3IndexMax then
								globEnmBullets3Index = 1
							end
		
							if globIsInDemoMode == false then 
								shotSnd:stop()
								shotSnd:play()
							end
					
					
						end
					end
				end
			end
	
		end
		
	else --spreadshot
		
		
		if self.core.active == true then --maybe could be enemyD active but core inactive, just to be sure
		
			local lclLaunchPX = self.core.x + 5 --the -3 because of the width of the shot
			local lclLaunchPY = self.core.y + 7

			local lclNumWays = 3 --default

			--may need to randomize numWays
			if levlBigShotMaxWays == 4 then
	
				local lclNumWaysRnd = love.math.random(100)
		
				if lclNumWaysRnd > 50 then
					lclNumWays = 4
				end
	
			elseif levlBigShotMaxWays == 5 then
	
				local lclNumWaysRnd = love.math.random(100)
		
				if lclNumWaysRnd <= 33 then
					lclNumWays = 4
				elseif lclNumWaysRnd >= 66 then
					lclNumWays = 5
				end
			end
	
			if lclNumWays == 3 or lclNumWays == 5 then
	
				--straight shot
				globEnmBullets3[globEnmBullets3Index].activate(globEnmBullets3[globEnmBullets3Index], lclLaunchPX, lclLaunchPY, 0.0, levlEnmBigBulSpdBasicY)
		
				globEnmBullets3Index = globEnmBullets3Index + 1
				if globEnmBullets3Index >= globEnmBullets3IndexMax then
					globEnmBullets3Index = 1
				end
		
				--first left shot
				globEnmBullets3[globEnmBullets3Index].activate(globEnmBullets3[globEnmBullets3Index], lclLaunchPX, lclLaunchPY, -levlEnmBigBulSpd3WayX, levlEnmBigBulSpdBasicY - levlEnmBigBulSpdSlowdorn3WayY)
		
				globEnmBullets3Index = globEnmBullets3Index + 1
				if globEnmBullets3Index >= globEnmBullets3IndexMax then
					globEnmBullets3Index = 1
				end
		
				--first right shot
				globEnmBullets3[globEnmBullets3Index].activate(globEnmBullets3[globEnmBullets3Index], lclLaunchPX, lclLaunchPY, levlEnmBigBulSpd3WayX, levlEnmBigBulSpdBasicY - levlEnmBigBulSpdSlowdorn3WayY)
		
				globEnmBullets3Index = globEnmBullets3Index + 1
				if globEnmBullets3Index >= globEnmBullets3IndexMax then
					globEnmBullets3Index = 1
				end
			end
	
			if lclNumWays == 5 then
	
				--second left shot
				globEnmBullets3[globEnmBullets3Index].activate(globEnmBullets3[globEnmBullets3Index], lclLaunchPX, lclLaunchPY, -levlEnmBigBulSpd5WayX, levlEnmBigBulSpdBasicY - levlEnmBigBulSpdSlowdorn5WayY)
		
				globEnmBullets3Index = globEnmBullets3Index + 1
				if globEnmBullets3Index >= globEnmBullets3IndexMax then
					globEnmBullets3Index = 1
				end
		
				--second right shot
				globEnmBullets3[globEnmBullets3Index].activate(globEnmBullets3[globEnmBullets3Index], lclLaunchPX, lclLaunchPY, levlEnmBigBulSpd5WayX, levlEnmBigBulSpdBasicY - levlEnmBigBulSpdSlowdorn5WayY)
		
				globEnmBullets3Index = globEnmBullets3Index + 1
				if globEnmBullets3Index >= globEnmBullets3IndexMax then
					globEnmBullets3Index = 1
				end
	
			end 
	
			if lclNumWays == 4 then
	
				--first left shot 
				globEnmBullets3[globEnmBullets3Index].activate(globEnmBullets3[globEnmBullets3Index], lclLaunchPX, lclLaunchPY, -levlEnmBigBulSpd4WayX, levlEnmBigBulSpdBasicY)
		
				globEnmBullets3Index = globEnmBullets3Index + 1
				if globEnmBullets3Index >= globEnmBullets3IndexMax then
					globEnmBullets3Index = 1
				end
		
				--first right shot
				globEnmBullets3[globEnmBullets3Index].activate(globEnmBullets3[globEnmBullets3Index], lclLaunchPX, lclLaunchPY, levlEnmBigBulSpd4WayX, levlEnmBigBulSpdBasicY)
		
				globEnmBullets3Index = globEnmBullets3Index + 1
				if globEnmBullets3Index >= globEnmBullets3IndexMax then
					globEnmBullets3Index = 1
				end
		
				--second left shot
				globEnmBullets3[globEnmBullets3Index].activate(globEnmBullets3[globEnmBullets3Index], lclLaunchPX, lclLaunchPY, (-3)*levlEnmBigBulSpd4WayX, levlEnmBigBulSpdBasicY - levlEnmBigBulSpdSlowdorn4WayY)
		
				globEnmBullets3Index = globEnmBullets3Index + 1
				if globEnmBullets3Index >= globEnmBullets3IndexMax then
					globEnmBullets3Index = 1
				end
		
				--second right shot
				globEnmBullets3[globEnmBullets3Index].activate(globEnmBullets3[globEnmBullets3Index], lclLaunchPX, lclLaunchPY, 3*levlEnmBigBulSpd4WayX, levlEnmBigBulSpdBasicY - levlEnmBigBulSpdSlowdorn4WayY)
		
				globEnmBullets3Index = globEnmBullets3Index + 1
				if globEnmBullets3Index >= globEnmBullets3IndexMax then
					globEnmBullets3Index = 1
				end
		
	
			end
		
			if globIsInDemoMode == false then 
				pumpShotSnd:stop()
				pumpShotSnd:play()
			end
		
		end
		
	end

end

function EnemyD.doDyingProcess(self)

	if self.dieCountDown == 0 then
		self.dieCountDown = 80
	end
	
	if self.dieCountDown > 0 then
	
		for i = 1,8,1 do 
			for j = 1,9,1 do
			
				if self.bricks[i][j].active == true then
				
					--if self.bricks[i][j].type ~= 2 then --make no exceptions for the bricks, leaving only brick 2 intact looks awkward
					
						--create placeholder graphic, always use brick 1 for template
						local locImageData = love.image.newImageData( 5, 3)
						locImageData:paste( enmBrickGph[1]:getData(), 0, 0, 0, 0, 5, 3 )
						self.bricks[i][j].currentGph = lg.newImage(locImageData)
						self.bricks[i][j].currentGph:setFilter("nearest","nearest")
						--recolor the placeholder graphic
						dat = self.bricks[i][j].currentGph:getData()
						dat:mapPixel( makeReColor3 )
						self.bricks[i][j].currentGph:refresh()
					--end
				end
			end
		end
	
		self.dieCountDown = self.dieCountDown - 1
		
		if self.dieCountDown == 1 then
		
			self.deactivate(self)
		end
	end

end




function EnemyD.doHorizontalMovement(self)
	
	
	if self.isRoaming == true then
		
		self.moveXCount = self.moveXCount + 1
		
		if self.moveXCount >= self.moveXMax then
			self.isRoaming = false
			self.isTurning = true
			
			self.nextSpeedX = 0.7 + love.math.random()*0.5
		end
		
	elseif self.isTurning == true then
		
		self.speedX = self.speedX - 0.05
		
		if self.speedX < (-1)*self.nextSpeedX then
			self.isRoaming = true
			self.isTurning = false
			
			self.speedXPresign = self.speedXPresign*(-1)
			self.speedX = self.nextSpeedX
			
			self.moveXCount = 0
			self.moveXMax = self.moveXBasicCnt + love.math.random(self.moveXRandomCnt)
		end
	end
	
	self.moveProgressX = self.moveProgressX + self.speedXPresign*self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	while self.moveProgressX >= threshold do --for negative x-Movement
		self.moveProgressX = self.moveProgressX - threshold
		self.x = self.x + 1
	end
	
	if self.x > self.boundsRight then
		self.x = self.boundsRight
		self.speedXPresign = self.speedXPresign*(-1)
	end
	
	if self.x < self.boundsLeft then
		self.x = self.boundsLeft
		self.speedXPresign = self.speedXPresign*(-1)
	end
	
	
	--actualize the position of the bricks and the core
	self.core.putPosition(self.core, self.x + self.corePosX, self.y + self.corePosY)
	
	for i = 1,8,1 do 
		for j = 1,9,1 do
		
			if self.bricks[i][j].active == true then--move only active bricks or else it will crash because of bricks boundingbox
				self.bricks[i][j].putPosition(self.bricks[i][j], self.x + self.bricks[i][j].column*5, self.y + self.bricks[i][j].row*3)
			end
		end
	end
	
end

function EnemyD.doVerticalMovement(self)


	if self.moveYUpsideDown == false then
		self.speedY = self.speedY - self.moveYSlowDown
	else
		self.speedY = self.speedY + self.moveYSlowDown
	end
	
	self.moveProgressY = self.moveProgressY - self.speedY
		
	while self.moveProgressY <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressY = self.moveProgressY + threshold
		self.y = self.y - 1 
	end
	
	while self.moveProgressY >= threshold do --for negative x-Movement
		self.moveProgressY = self.moveProgressY - threshold
		self.y = self.y + 1 
	end
	
	if self.moveYUpsideDown == false and self.y > self.startPosY then
		self.moveYUpsideDown = true
		self.speedY = (-1)*self.moveYBasic
	end
	
	if self.moveYUpsideDown == true and self.y < self.startPosY then
		self.moveYUpsideDown = false
		self.speedY = self.moveYBasic
	end
	
	--actualize the position of the bricks and the core
	self.core.putPosition(self.core, self.x + self.corePosX, self.y + self.corePosY)
	
	for i = 1,8,1 do 
		for j = 1,9,1 do
		
			if self.bricks[i][j].active == true then--move only active bricks or else it will crash because of bricks boundingbox
				self.bricks[i][j].putPosition(self.bricks[i][j], self.x + self.bricks[i][j].column*5, self.y + self.bricks[i][j].row*3)
			end
		end
	end

end

function EnemyD.prepareShoot(self)
	print("Bot Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end

function EnemyD.draw(self)

	if self.active == true or self.semiActive == true then	
		
		self.core.draw(self.core)
		
		for i = 1,8,1 do 
			for j = 1,9,1 do
				self.bricks[i][j].draw(self.bricks[i][j])
			end
		end
	end
end

--function EnemyD.adjustBoundingBox(self)

	--1 frame, 1 box, simplified version
--	self.currentBBox.doUpdate(self.currentBBox, self.x, self.y)
--end







