require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyE = Object:extend()


function EnemyE.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyE:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	self.currentBBox = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.speedPresign = 0
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	self.speedY = 0.0
	self.moveProgressY = 0.0
	
	self.magnitudeY = 0.0
	self.magnitudeYRnd = 0.0
	self.magnitudeX = 0.0
	self.magnitudeXRnd = 0.0
	self.ySlowDown = 0.0
	
	self.bounceOffCounter = 0
	self.bounceBoundsLeft = 0
	self.bounceBoundsRight = 193
	self.bounceBoundsBottom = 117

	self.isCrawling = false
	self.isJumping  = false
	
	self.crawlSpd = 0.0
	self.crawlProgress = 0.0
	
	self.primaryAnmIndex = 0
	
	self.crawlAnmStep = 0 --ineffective value
	self.crawlAnmProgress = 0
	self.crawlAnmThresh = 4
	self.crawlAnmIndex = 0 --invalid value
	self.crawlAnmIndexMax = 4
	
	self.jumpAnmStep = 0 --ineffective value
	self.jumpAnmProgress = 0
	self.jumpAnmThresh = 8
	self.jumpAnmIndex = 0 --invalid value
	self.jumpAnmIndexMax = 4
end

function EnemyE.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyE.setMagnitudeY(self, aMagn, aMagnRnd)
	self.magnitudeY = aMagn
	self.magnitudeYRnd = aMagnRnd
end

function EnemyE.setMagnitudeX(self, aMagn, aMagnRnd)
	self.magnitudeX = aMagn
	self.magnitudeXRnd = aMagnRnd
end

function EnemyE.setYSlowdown(self, aYSlow)
	self.ySlowDown = aYSlow
end

function EnemyE.activate(self)
	
	self.active = true

	self.scoreReward = 80
	
	self.enmWidth  = 7
	self.enmHeight = 7
	
	self.hitPoints = 1
	
	self.isCrawling = true
	self.isJumping  = false
	
	self.crawlSpd = 0.65
	
	self.primaryAnmIndex = 1 --valid value
	
	self.crawlAnmStep = 1 --effective value
	self.crawlAnmIndex = 1 --valid value
	
	self.jumpAnmStep = 1 --effective value
	self.jumpAnmIndex = 1 --valid value
	
	self.currentGph  = enemyEGph[self.primaryAnmIndex][self.crawlAnmIndex]
	self.currentBBox = BoundingBox(enemyEBoxGph)
	
	self.adjustBoundingBox(self)
	
end

function EnemyE.plantExplosion(self)

	--do nothing
	
end

function EnemyE.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyE.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyE.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				
				if globIsInDemoMode == false then 
					enemyHit2Snd:stop()
					enemyHit2Snd:play()
				end
				
				self.deactivate(self)
			end
		end
	end
	
	--only if still active
	if self.active == true then
	
		if self.isCrawling == true then
			self.doCrawlAnimation(self)
			self.executeCrawlMove(self)
		end
		
		if self.isJumping == true then
			self.doJumpAnimation(self)
			self.executeJumpMove(self)
		end
	end
end

function EnemyE.doCrawlAnimation(self)

	self.crawlAnmProgress = self.crawlAnmProgress + self.crawlAnmStep
	
	if self.crawlAnmProgress >= self.crawlAnmThresh then
		self.crawlAnmProgress = 0
		
		self.crawlAnmIndex = self.crawlAnmIndex + 1
		
		if self.crawlAnmIndex > self.crawlAnmIndexMax then
			self.crawlAnmIndex = 1
		end
		
		self.currentGph  = enemyEGph[self.primaryAnmIndex][self.crawlAnmIndex]
	end
	
end

function EnemyE.executeCrawlMove(self)

	self.crawlProgress = self.crawlProgress + self.crawlSpd
	
	if self.crawlProgress >= threshold then
		self.crawlProgress = self.crawlProgress - threshold
		
		self.y = self.y - 1
	end
	
	self.adjustBoundingBox(self)
	
	if self.y <= self.bounceBoundsBottom then --enemy reach top of ground
	
		--switch to jumping
		self.isCrawling = false
		self.isJumping  = true
		self.primaryAnmIndex = 2
	end
	
end

function EnemyE.doJumpAnimation(self)
	
	self.jumpAnmProgress = self.jumpAnmProgress + self.jumpAnmStep
	
	if self.jumpAnmProgress >= self.jumpAnmThresh then
		self.jumpAnmProgress = 0
		
		self.jumpAnmIndex = self.jumpAnmIndex + 1
		
		if self.jumpAnmIndex > self.jumpAnmIndexMax then
			self.jumpAnmIndex = 1
		end
		
		self.currentGph  = enemyEGph[self.primaryAnmIndex][self.jumpAnmIndexMax]
	end
	
end

function EnemyE.executeJumpMove(self)

	if self.speedPresign == 0 then
		--randomize first jump direction and jumping speedPresign
		local lclFirstDirRnd = love.math.random(100)
		
		if lclFirstDirRnd <= 50 then
			self.speedPresign = (-1)
		else
			self.speedPresign = 1
		end
		
		--randomize first speed
		
		self.speedX = self.magnitudeX + love.math.random()*self.magnitudeXRnd
		
	end

	self.moveProgressX = self.moveProgressX + self.speedPresign*self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	while self.moveProgressX >= threshold do --for negative x-Movement
		self.moveProgressX = self.moveProgressX - threshold
		self.x = self.x + 1
	end
	
	if self.x <= self.bounceBoundsLeft then
		self.speedPresign = 1
	end

	if self.x >= self.bounceBoundsRight then
		self.speedPresign = (-1)
	end

	

	self.speedY = self.speedY - self.ySlowDown
		
	self.moveProgressY = self.moveProgressY - self.speedY
		
	while self.moveProgressY <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressY = self.moveProgressY + threshold
		self.y = self.y - 1
	end
	
	while self.moveProgressY >= threshold do --for negative x-Movement
		self.moveProgressY = self.moveProgressY - threshold
		self.y = self.y + 1
	end
	
	if self.y >= self.bounceBoundsBottom then
		self.speedY = self.magnitudeY + love.math.random()*self.magnitudeYRnd
		
		--randomize next xspeed
		
		self.speedX = self.magnitudeX + love.math.random()*self.magnitudeXRnd
	end
	
	
	self.adjustBoundingBox(self)

	
	
end

function EnemyE.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end

function EnemyE.adjustBoundingBox(self)

	--1 frame, 1 box, simplified version
	self.currentBBox.doUpdate(self.currentBBox, self.x, self.y)
end






