require "luas.shortcuts"
require "luas.extendedLoader"

ENMBULLET_INACTIVE   = 0	--type could be completely different projectiles in 1 class (or do different classes)
ENMBULLET_ACTIVE     = 1	--for this simple example, just active/inactive


EnmBullet1 = Object:extend()


function EnmBullet1.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:resetType()
	self:resetAnimation()
	
	--simplified, just 1 box (is used for all basic enemy bullet types here if there are types)
	--self.reservedBBox = BoundingBox(enmShotBoxGph)--not used
	
	self.reservedGphs = {}
	self.reservedGphs[1] = enmShotGph[1][1]
	self.reservedGphs[2] = enmShotGph[1][2]

end


function EnmBullet1:resetType()

	self.type = ENMBULLET_INACTIVE
	--following will be determined by type that is given on activation
	self.width = 0
	self.height= 0
	--self.currentBBox       = nil--not used
	self.currentGph        = nil
	
	self.x = 0
	self.y = 0
	
	self.xSpeedStep     = 0.0
	self.xSpeedProgress = 0.0
	self.xSpeedThresh= threshold
	
	self.ySpeedStep     = 0.0
	self.ySpeedProgress = 0.0
	self.ySpeedThresh= threshold
	
	--boundaries (basic values, getting fine adjustment according to bulletsize below)
	self.boundsBtm   = 150 - 25 - 4 --screen height minus groundthickness minus shotheight

end


function EnmBullet1:resetAnimation()
	--index for animations
	self.anmIndex          = 0--to invalid value
	self.anmMaxIndex       = 2--four frames
	--stepper variables for animation
	self.anmStep     = 0--to invalid value
	self.anmProgress = 0 --increase per update, start with a standstill (negative value)
	self.anmThresh   = 10  
	--self.anmTrigger  = 0 --throw that out, simplify
end


--may add bullet-type to parameterlist or do different classes for different bullets. 
--
function EnmBullet1.activate(self, aPX, aPY, aSX, aSY) --need loopdirection

	
	self.type = ENMBULLET_ACTIVE --SIMPLIFIED only one bullettype, activate
	
	--following will be determined by type that is given on activation
	self.width = 3 --for other shooters there could be variation between bullettype sizes, here it isn't
	self.height= 4
	
	
	--to valid value
	self.anmIndex = 1 

	self.anmStep  = 1
	
	--self.currentBBox = self.reservedBBox --not used
	self.currentGph  = self.reservedGphs[self.anmIndex] --to valid graphic
		
	--set bullet position
	self.x = aPX
	self.y = aPY
		
	--not used
	--self.adjustBoundingBox(self)
		
	--set bullet speeds
	self.xSpeedStep     = aSX
	self.ySpeedStep     = aSY
	
end


function EnmBullet1:deactivate()
	--set inactive, remove current-fields of boundingbox, sizes, graphics ect
	
	--set defaults
	self:resetType()
	self:resetAnimation()
end

function EnmBullet1.doUpdate(self)

	
	if self.type ~= ENMBULLET_INACTIVE then --when bullet not inactive

		--##############################
		--# calculate bullet animation
		--##############################
		
		self.anmProgress = self.anmProgress + self.anmStep
		--
		--only 1 frame forward or backward at a time, no while loop.
		--
		if self.anmProgress >= self.anmThresh then 	--forward boundary
		
			self.anmProgress = 0
			
			self.anmIndex = self.anmIndex + 1
			
			if self.anmIndex > self.anmMaxIndex then
				self.anmIndex = 1
			end
		end
		
		--##############################
		--# apply bullet animation
		--##############################
		
		self.currentGph = self.reservedGphs[self.anmIndex] --SIMPLIFIED, no bullet types
		
		
		--##############################
		--# bullet movements
		--##############################
		
		--bullet movement in x-directions
		self.xSpeedProgress = self.xSpeedProgress + self.xSpeedStep
		while self.xSpeedProgress >= self.xSpeedThresh do --for positive x-Movement
			self.xSpeedProgress = self.xSpeedProgress - self.xSpeedThresh
			self.x = self.x + 1
		end
		while self.xSpeedProgress <= ((-1) * self.xSpeedThresh) do --for negative x-Movement
			self.xSpeedProgress = self.xSpeedProgress + self.xSpeedThresh
			self.x = self.x - 1
		end
		--bullet movements in y directions 
		self.ySpeedProgress = self.ySpeedProgress + self.ySpeedStep 
		
		while self.ySpeedProgress >= self.ySpeedThresh do --for positive y-Movement (not used in this game)
			self.ySpeedProgress = self.ySpeedProgress - self.ySpeedThresh
			self.y = self.y + 1
		end
		while self.ySpeedProgress <= ((-1) * self.ySpeedThresh) do --for negative y-Movement (standard case)
			self.ySpeedProgress = self.ySpeedProgress + self.ySpeedThresh
			self.y = self.y - 1
		end
		
		--##############################
		--# after movement adjust bounding box
		--##############################
		
		--self.adjustBoundingBox(self)--not used
		
		
		--##############################
		--# bullet deactivation on boundaries
		--##############################
		
		if self.type ~= ENMBULLET_INACTIVE then --only check when still active to avoid mishaps
		
			if self.y > self.boundsBtm then
				self:deactivate()
			end
		end
		
		
	end
	
	--for collision-handling: see gamestates!

end

--function EnmBullet1.adjustBoundingBox(self)
--
--	--just in case include all boundingboxes (although bullet should not change during lifecycle)
--	--for i= 1,3,1 do
--		self.reservedBBox.doUpdate(self.reservedBBox, self.x, self.y)
--	--end
--end


function EnmBullet1.draw(self)
	if self.type ~= ENMBULLET_INACTIVE then
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
		
		--debug
		--self.currentBBox.draw(self.currentBBox)
	end
end


function EnmBullet1:CollisionCheck(aBox)

	if self.type ~= ENMBULLET_INACTIVE then
		if self.currentBBox == nil then
			return false
		else
			return self.currentBBox:CollisionTest(aBox)
		end
	else 
		return false
	end
end
