require "luas.shortcuts"
require "luas.boundingBox"
require "luas.extendedLoader"

PLBULLET_INACTIVE   = 0
PLBULLET_ACTIVE     = 1 --simplified, only active and inactive, no bullet types

PlayerBullet = Object:extend()


function PlayerBullet.new(self)	
	
	self.bulletNumColor = 0 --undefined
	
	
	--TODO careful: those functions will do less here than they do in a more advanced shmup
	self:resetType()
	
	--simplified, just 1 box
	self.reservedBBox = BoundingBox(playerShotBoxGph)

	
	self.currentGph = nil
	
	
end


--initialize bullet with color of corresponding player
function PlayerBullet.defineColorType(self, aColorNum) --multiplayer needs different shipcolors
	
	--define color
	self.bulletNumColor = aColorNum
	
	--apply color
	self.currentGph = playerShotGph[self.bulletNumColor]

 
end


function PlayerBullet:resetType()

	--no need to reset bullet color (corresponding to player) once it is initialized
	--self.bulletNumColor

	self.type = PLBULLET_INACTIVE
	--following will be determined by type that is given on activation
	self.width = 0
	self.height= 0
	self.currentBBox       = nil
	self.currentGph        = nil
	
	self.x = 0
	self.y = 0
	
	self.xSpeedStep     = 0.0
	self.xSpeedProgress = 0.0
	self.xSpeedThresh= threshold
	
	self.ySpeedStep     = 0.0
	self.ySpeedProgress = 0.0
	self.ySpeedThresh= threshold
	
	--boundaries (basic values, getting fine adjustment according to bulletsize below)
	self.boundsLeft  = 0
	self.boundsRight = 200
	self.boundsTop   = 0
	self.boundsBtm   = 150

end


--simplified, in this game: no different bullet types as to how strong the bullets are.
function PlayerBullet.activate(self, aPX, aPY, aSX, aSY)--, anmOffset)--SIMPLIFIED no animation offset and different bullettypes
		
	self.type = PLBULLET_ACTIVE --SIMPLIFIED only one bullettype
	
	--following will be determined by type that is given on activation
	self.width = 3 --for other shooters there could be variation between bullettype sizes, here it isn't
	self.height= 6
		
	self.boundsLeft  = 0
	self.boundsRight = 200
	self.boundsTop   = (-6)
	self.boundsBtm   = 150
		
	self.currentBBox = self.reservedBBox --SIMPLIFIED only 1 box to access
	
	self.currentGph = playerShotGph[self.bulletNumColor] --is set to nil in deactivation so it needs to be re-set explicitly
		
	--set bullet position
	self.x = aPX
	self.y = aPY
		
	--first boundingbox adjustment right on activation, otherwise it will sit on wrong place for 1 frame
	self.adjustBoundingBox(self)
		
	--set bullet speeds
	self.xSpeedStep     = aSX
	self.ySpeedStep     = aSY
	
end


function PlayerBullet:deactivate()
	--set inactive, remove current-fields of boundingbox, sizes, graphics ect
	
	--set defaults
	self:resetType()
	--self:resetAnimation() --no animation
end


function PlayerBullet.doUpdate(self)

	--##############################
	--# calculate bullet animation
	--##############################
	
	if self.type ~= PLBULLET_INACTIVE then --when bullet not inactive

		
		--##############################
		--# apply bullet animation
		--##############################
		
		--self.currentGph = self.Gphs[self.anmIndex] --no animation, graphic set on activation
		
		
		--##############################
		--# bullet movements
		--##############################
		
		--bullet movement in x-directions
		self.xSpeedProgress = self.xSpeedProgress + self.xSpeedStep
		while self.xSpeedProgress >= self.xSpeedThresh do --for positive x-Movement
			self.xSpeedProgress = self.xSpeedProgress - self.xSpeedThresh
			self.x = self.x + 1
		end
		while self.xSpeedProgress <= ((-1) * self.xSpeedThresh) do --for negative x-Movement
			self.xSpeedProgress = self.xSpeedProgress + self.xSpeedThresh
			self.x = self.x - 1
		end
		--bullet movements in y directions (NOT USED IN THIS GAME)
		self.ySpeedProgress = self.ySpeedProgress + self.ySpeedStep 
		
		while self.ySpeedProgress >= self.ySpeedThresh do --for positive y-Movement (not used in this game)
			self.ySpeedProgress = self.ySpeedProgress - self.ySpeedThresh
			self.y = self.y + 1
		end
		while self.ySpeedProgress <= ((-1) * self.ySpeedThresh) do --for negative y-Movement (standard case)
			self.ySpeedProgress = self.ySpeedProgress + self.ySpeedThresh
			self.y = self.y - 1
		end
		
		--##############################
		--# after movement adjust bounding box
		--##############################
		
		self.adjustBoundingBox(self)
		
		
		--##############################
		--# bullet deactivation on boundaries
		--##############################
		
		if self.x < self.boundsLeft then
			self:deactivate()
		end
		if self.x > self.boundsRight then
			self:deactivate()
		end
		if self.y < self.boundsTop then
			self:deactivate()
		end
		if self.y > self.boundsBtm then
			self:deactivate()
		end
		
		
	end
	
	--for collision-handling: see gamestates!

end


function PlayerBullet.adjustBoundingBox(self)

	--just in case include all boundingboxes (although bullet should not change during lifecycle)
	--for i= 1,3,1 do
		self.reservedBBox.doUpdate(self.reservedBBox, self.x, self.y)
	--end
end


function PlayerBullet.draw(self)
	if self.type ~= PLBULLET_INACTIVE then
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
		
		--debug
		--self.currentBBox.draw(self.currentBBox)
	end
end

--removed in favor of point collision test
--
--function PlayerBullet:CollisionCheck(aBox)

--	if self.type ~= PLBULLET_INACTIVE then
--		if self.currentBBox == nil then
--			return false
--		else
--			return self.currentBBox:CollisionTest(aBox)
--		end
--	else
--		return false
--	end
--end

function PlayerBullet:CollisionCheck(aPx, aPy)

	if self.type ~= PLBULLET_INACTIVE then
		if self.currentBBox == nil then
			return false
		else
			return self.currentBBox:InBoxTest(aPx, aPy)
		end
	else
		return false
	end
end








