require "luas.shortcuts"
require "luas.boundingBox"
require "luas.extendedLoader"
require "luas.playerBullet"
--require "luas/globals"  --player can't require globals because globals requires player and that creates
                          --a circle-require (loop). but globals initializes variables globally with 
						  --static function calls so accessing the stuff from globals works anyway

PlayerShip = Object:extend()

function PlayerShip.new(self)

	self.active = false

	self.enteredGame = false
	
	self.inputType = GLOB_INPUT_WAITING

	self.playerNumColor = 0           --undefined (define in defineColorType)
	
	self.respawnedVulnerableFlag = false
	self.RespawnCountDownIndex   = 0  --standard: player starts invulnerable for short period of time, only for 2nd and 3rd player, see below
	self.RespawnCountDownMax     = 140
	self.RespawnCountDownResume  = 1
	
	self.shipWidth  = 13 
	self.shipHeight = 10 
	
	self.currentBBox = BoundingBox(shipBoxGph)
	self.currentMouseBox = BoundingBox(mouseBoxGph) --boundingbox-like for directional mouse control
	
	self.currentGph = nil --undefined, needs to be defined with defineColor
	self.isVisible  = true

	self.moveSpeed      = 2.5
	self.dx			    = 0.0 --movement restricted to x
	
	self.x              = globPlayerStartPosX
	self.y              = globPlayerStartPosY
	
	--adjust boundingbox right at the beginning or it will
	--be at the wrong place in the first frame
	
	self.adjustFunctionBoxes(self) --does self.adjustBoundingBox(self) + self.adjustMouseBox(self) --same for mousebox
	
	--may introduce adjustFunctionBoxes
	
	self.boundsLeft  = globPlayerLeftBounds
	self.boundsRight = globPlayerRightBounds
	
	self.leadPlBulletStep     = 1
	self.leadPlBulletProgress = 12 --will be able to shoot
	self.leadPlBulletThresh   = 12
	self.leadPlBulletMax      = 2 
	self.shootCondition1=false
	self.shootCondition2=false
	
	self.leftPress  = false
	self.rightPress = false
	self.firePress  = false
	
	print("Playership Constructor Call")
	
	
	--self.debugRightMouseWasDown= false
	
	--score in this case is not for the whole game but for the individual 
	--player. It is a score counted from jointime onward used to calculate extra lifes
	self.playerScorePrevious = 0
	self.playerScoreCurrent  = 0
	self.SCORE_EXTRALIFE_1   = 5000
	self.SCORE_EXTRALIFE_2   = 10000
	
	--frame for toggling of sprite visibility
	self.ToggleFrame = {}
	self.ToggleFrameIndex    = 1
	self.ToggleFrameIndexMax = 8 
	--for i=1,self.ToggleFrameIndexMax,1 do		--dont use loop,set manually
	self.ToggleFrame[1] = true
	self.ToggleFrame[2] = true
	self.ToggleFrame[3] = false
	self.ToggleFrame[4] = false
	self.ToggleFrame[5] = false
	self.ToggleFrame[6] = false
	self.ToggleFrame[7] = true
	self.ToggleFrame[8] = true
	--end
	
	--self.deathAnmProgress = 0 -- default: handle death animation by yPosition of player, offset determined here
	
	self.dyingCountDown = 0
	self.dyingCountDownMax = 180--240
	self.doResumeChecks = false
	
	self.demoMoveDir = 0
end

function PlayerShip.getCurrentInputType(self)

	return self.inputType
end

function PlayerShip.addScore(self, aScorePlus)
	
	if self.enteredGame == true and self.active == true then
	
		if self.inputType ~= GLOB_INPUT_COMPUTER then--prevent adding score for computer-controlled players aka demoMode
	
			self.playerScoreCurrent = self.playerScoreCurrent + aScorePlus
			
			if self.playerScoreCurrent > globMaxScore then
				self.playerScoreCurrent = globMaxScore
			end
	
			--hardwire for gaining a maximum of 2 additional lifes with score
			if self.playerScorePrevious < self.SCORE_EXTRALIFE_1 and self.playerScoreCurrent >= self.SCORE_EXTRALIFE_1 then
				globNumLifes[self.playerNumColor] = globNumLifes[self.playerNumColor] + 1
				
				if globIsInDemoMode == false then 
					extraLifeSnd:stop()
					extraLifeSnd:play()
				end
			end
			if self.playerScorePrevious < self.SCORE_EXTRALIFE_2 and self.playerScoreCurrent >= self.SCORE_EXTRALIFE_2 then
				globNumLifes[self.playerNumColor] = globNumLifes[self.playerNumColor] + 1
				
				if globIsInDemoMode == false then 
					extraLifeSnd:stop()
					extraLifeSnd:play()
				end
			end
			
		
			self.playerScorePrevious = self.playerScoreCurrent
		end
	end
	

end

function PlayerShip.defineColorType(self, aColorNum) --multiplayer needs different shipcolors
	
	--define color
	self.playerNumColor = aColorNum
	--apply color
	self.currentGph = playerShipGph[self.playerNumColor]
	
	print("Player is defined, PlayerNumber "..self.playerNumColor)
 
end

function PlayerShip.enterGame(self, aInputType)
	
	print("Player enters game:")
	print("Player number is "..self.playerNumColor)
	
	
	if self.playerNumColor == 1 then --color of first player
		self.respawnedVulnerableFlag = true --first player starts the game instantly vulnerable(as opposed to 
		                                    --those who join later and may otherwise be unfairly placed on a bullet)
		self.RespawnCountDownIndex   = 0
	end
	
	if self.playerNumColor <= 0 then
		--must not happen, need playercolor aka playernumber 
		--before player can enter and inputdevice be set
		dummy=1/0
	end
	
	if aInputType ~= GLOB_INPUT_WAITING then --playernumber aka color must be defined and input-type not passive...
		self.enteredGame = true
		self.inputType = aInputType
		--on entering game, always activate. 
		self.activate(self)
	
	elseif self.playerNumColor == 1 and aInputType == GLOB_INPUT_WAITING then --must not happen, 
		--because player 1 needs a valid input-device at this point. other players may join later.
		dummy = 1/0 
	end
	
	local debugVul = 0
	if self.respawnedVulnerableFlag ==true then
		debugVul = 1
	end
	local debugEnt = 0
	if self.enteredGame ==true then
		debugEnt = 1
	end
	print("Initial Vulnerability is "..debugVul.." Vulnerability Countdown starts with "..self.RespawnCountDownIndex)
	print("Player self.enteredGame is "..debugEnt..", input Type is "..self.inputType)
	
end

function PlayerShip.activate(self)
	if self.enteredGame then
		self.active = true
	else
		--player not supposed to get activated if not entered game
		dummy = 1/0
	end
end

function PlayerShip.deactivate(self)
	self.active = false
	--TODO consider reset function, not essential in this game because player can't recover from gameover
	--and will be completely re-instantiated with gamestate change so unlike other objects, no re-activation.
	--
	--also ALWAYS keep self.enteredGame true on deactivation even with reset which means player had its chance
end

function PlayerShip.setDeathAnmProgress(self, aAnmProgress)
	--could as well be real death animation such as explosion, 
	--here it is a sink-into-ground Progress
	
	--no, don't mix up update action with the action that sets the parameter
	--self.y = globPlayerStartPosY - aAnmProgress
	
	self.deathAnmProgress = aAnmProgress
	
end

--function PlayerShip.doSmallUpdate(self) --not for death animation which is handled by gamestate

--	self.y = globPlayerStartPosY - self.deathAnmProgress
	
	
	--if self.deathAnmProgress == 1 then
	--	print(" S   M   A   L   L            U   P   D   A   T   E    1")
	--elseif self.deathAnmProgress == 2 then
	--	print(" S   M   A   L   L            U   P   D   A   T   E    2")
	--end
	
--end

function PlayerShip.doUpdate(self)


	local lclBlockFireInput     = false --ignore fire button in this update (?)
	local lclSpecialUpdateOnly = false --ignore most actions in this update (?)

	if self.dyingCountDown > 0 then
		lclSpecialUpdateOnly = true
		
		--print("Player "..self.playerNumColor.." dying - block most update")
	end

	--TODO entryMute has to be transferred to player (2nd and 3rd player should 
	--also start out not instantly shooting but shooting of previously active player(s) must not be blocked)
	if globEntryMuteCountDownIndex > 0 and globGameOverMuteCountDownIndex > 0 then
		lclBlockFireInput = true
		
		--print("Player "..self.playerNumColor.." entry mute - block fire")
	end
	
	if self.enteredGame == false then --if not yet entered game, ...

		if self.playerNumColor > 1 then --does not apply to player 1 but to other players joining on the run
		
			--TODO...listen for firebutton to enable entering the game AND determining input device for this ship
			if lclSpecialUpdateOnly == false then --...but only if game isnt blocked by other player's death
				--TODO set self.enteredGame
				--TODO set input device
			end
		end
		
	
	else --enteredGame true, update other than waiting for game-enter...
		
		
		
		--#############################################
		--#  For Playership after dying do resumechecks
		--#############################################
				
		if self.doResumeChecks == true then
			
			self.doResumeChecks = false --only do the check once
				
			--self.playerGameOver
			--self.playerRespawns
			if globNumLifes[self.playerNumColor] > 0 then
			
				if globIsInDemoMode == false then--don't lose lifes when game is in demomode
					globNumLifes[self.playerNumColor] = globNumLifes[self.playerNumColor] - 1 --one life less 
				end
				
				self.respawnedVulnerableFlag = false
				self.isVisible = true --make re-visible
				self.active = true
				
			else
				globNumLifes[self.playerNumColor] = (-1) --player gameover
				
				print("Player game over")
			end
				
		end
		
		
		
		--with entered game flag set
		
		if lclSpecialUpdateOnly == false then
		
			if self.active == true then --as long as active
			
		
				if globNumLifes[self.playerNumColor] < 0 then --check if lifes used up, if so deactivate
			
					self.deactivate(self) --in next loop will not be active and will neither do normal nor special update part
			
				else
			
				--###################################################################################
				--# known in this inner part : player is in game and active
				--###################################################################################
				--# let player do the real part of this update or the special part of death animation
				--###################################################################################
			
			
				--if lclSpecialUpdateOnly == false then --doesnt work here, move it up
				
				
					--####################################
					--#  Respawn Vulnerability Handling  #
					--####################################
		
					if self.respawnedVulnerableFlag == false and self.RespawnCountDownIndex == 0 then
					
						--respawn flag at invulnerable and respawn countdown inactive, set countdown
						self.RespawnCountDownIndex = self.RespawnCountDownMax
						--self.respawnedVulnerableFlag = false
						--self.RespawnCountDownIndex   = 140  --standard: player starts invulnerable for short period of time
						--self.RespawnCountDownMax     = 140
						--self.RespawnCountDownResume  = 1
						
						
					elseif self.RespawnCountDownIndex > self.RespawnCountDownResume then
					
						print("self.RespawnCountDownIndex "..self.RespawnCountDownIndex)
					
						--run countdown ...
						self.RespawnCountDownIndex = self.RespawnCountDownIndex - 1
						
						--and let it toggle visibility
						self.ToggleFrameIndex = self.ToggleFrameIndex + 1
						if self.ToggleFrameIndex == self.ToggleFrameIndexMax then
							self.ToggleFrameIndex = 1
						end
						self.isVisible = self.ToggleFrame[self.ToggleFrameIndex]
						
					elseif self.RespawnCountDownIndex == self.RespawnCountDownResume then
					
						--end countdown and reset respawn flag 
						self.RespawnCountDownIndex = 0
						self.respawnedVulnerableFlag = true
						
						--restore visibility
						self.isVisible = true
					else
						--keep or restore visibility
						self.isVisible = true
					end
				
				
				
					if self.inputType ~= GLOB_INPUT_COMPUTER then --for all inputs other than computer simultaneously
					--if self.inputType == GLOB_INPUT_KEYBOARD then --react to all inputs simultaneously
						
						if lkeyBrd("left") == true then
							self.leftPress  = true
						end
						if lkeyBrd("right") == true then
							self.rightPress = true
						end
						if lclBlockFireInput == false then
							if lkeyBrd("z") == true or lkeyBrd("y") then
								self.firePress  = true
							end
						end
				
						--if self.leftPress == true then
						--	print("Player "..self.playerNumColor.." notice keyboard-left press")
						--end
						--if self.rightPress == true then 
						--	print("Player "..self.playerNumColor.." notice keyboard-right press")
						--end
						--if self.firePress == true then
						--	print("Player "..self.playerNumColor.." notice keyboard-fire press")
						--end
	
					--elseif self.inputType == GLOB_INPUT_GAMEPAD then--react to all inputs simultaneously
			
						if globJoystick then --only react when joystick 1 is actually in usb 
							if globJoystick:isGamepadDown("dpleft") == true then
								self.leftPress  = true
							end
							if globJoystick:isGamepadDown("dpright") == true then
								self.rightPress = true
							end
					
							if lclBlockFireInput == false then
						
								--down10 = globJoystick:isDown(10)
								--down11 = globJoystick:isDown(11)
								--down12 = globJoystick:isDown(12)
								--down13 = globJoystick:isDown(13)
								--if globJoystick:isDown(10) or globJoystick:isDown(11) or globJoystick:isDown(12) or globJoystick:isDown(13) then
								--	self.firePress  = true
								--end
								
								local lclGpadFirePress = globJoystick:isGamepadDown("a", "b", "x", "y")
								
								if lclGpadFirePress == true then
									self.firePress  = true
								end
						
							end
						end
						
					--elseif self.inputType == GLOB_INPUT_GAMEPAD2 then --all inputs simultaneously
					
						if globJoystick2 then --only react when joystick 1 is actually in usb 
					
							if globJoystick2:isGamepadDown("dpleft") == true then
								self.leftPress  = true
							end
							if globJoystick2:isGamepadDown("dpright") == true then
								self.rightPress = true
							end
					
							if lclBlockFireInput == false then
								local lclGpadFirePress = globJoystick2:isGamepadDown("a", "b", "x", "y")
								
								if lclGpadFirePress == true then
									self.firePress  = true
								end
						
							end
						end
					
					--elseif self.inputType == GLOB_INPUT_GAMEPAD3 then --all inputs simultaneously
					
						if globJoystick3 then --only react when joystick 1 is actually in usb 
							if globJoystick3:isGamepadDown("dpleft") == true then
								self.leftPress  = true
							end
							if globJoystick3:isGamepadDown("dpright") == true then
								self.rightPress = true
							end
					
							if lclBlockFireInput == false then
								
								local lclGpadFirePress = globJoystick3:isGamepadDown("a", "b", "x", "y")
								
								if lclGpadFirePress == true then
									self.firePress  = true
								end
						
							end
						end
	
					--elseif self.inputType == GLOB_INPUT_MOUSE then --all inputs simultaneously
			
						--move ship with right mouse button down relative to playership (and its mouse-box)
						local leftRightCheck = lMouseDown("r") --fire with left mousebutton down 
				
						--if leftRightCheck == true then
							--print("Right mouse btn down is true")
						--else
							--print("Right mouse down is false")
						--end
				
				
						if leftRightCheck == true then
				
							--for mousepointer left of mousebox trigger movement to the left
							--for mousepointer right of mousebox trigger movement to the right
					
							--problem here: mouseposition is absolute window position.
							--whereas player position is canvas position. need to get transformed mouse position
							--in canvas coordinates. calculating those should not be the business of the playerobject
							--likely put in globals.
					
							local lclMouseX = lMouseGetX()
							--transfrom mouse according to game resolution (window to big-pixel)
							lclMouseX = mouseX_Transformed(lclMouseX)
					
							--print("MouseX is"..lclMouseX)
							--print("MouseBoxMinX is "..self.currentMouseBox.minXT)
							--print("MouseBoxMaxX is "..self.currentMouseBox.maxXT)
					
							if lclMouseX < self.currentMouseBox.minXT then
								self.leftPress = true
								--print("Moving left")
							elseif lclMouseX > self.currentMouseBox.maxXT then
								self.rightPress = true
								--print("Moving right")
							else
								--print("Moving not!")
							end
						end
				
				
						if lclBlockFireInput == false then
	
							if lMouseDown("l") == true then
								self.firePress  = true --fire with left mousebutton down 
							end
							--if self.firePress ==true then
								--print("Left mouse btn down is true")
							--end
						end
				

					else --elseif self.inputType == GLOB_INPUT_COMPUTER then	--globinputcomputer is simply the else case here
						
						if self.demoMoveDir == 0 then--ask demoMovedirection for first time in game. 0 is undefined so force a direction
							local rnd =love.math.random(100)
			
							if rnd <= 50 then
								self.demoMoveDir = 1
							else
								self.demoMoveDir = 2
							end
						end
						--apply old direction
						if self.demoMoveDir == 1 then--in process of moving left
							self.rightPress = false
							self.leftPress = true
						elseif self.demoMoveDir == 2 then--in process of moving right
							self.rightPress = true
							self.leftPress = false
						elseif self.demoMoveDir == 3 then--in process of standing still
							self.rightPress = false
							self.leftPress = false
						end
		
						--possible directionchange for next update by chance
						local rndDirChange =love.math.random(1000)
		
						if rndDirChange < 50 then
							self.demoMoveDir = 1 --change to moving left
						elseif rndDirChange > 950 then
							self.demoMoveDir = 2 --change to moving right
						elseif rndDirChange > 500 and rndDirChange < 550 then
							self.demoMoveDir = 3 --change to standing still
						end
						
						self.firePress  = true --have computer-controlled player always press fire
						
					end
		
					--add to left right movement
					if self.rightPress == true and self.leftPress == true then --TODO also in other projects:
						-- do not react to contradicting input, otherwise with two directions it will pick one at random
					elseif self.rightPress == true then 
						self.dx	= self.dx + self.moveSpeed
					elseif self.leftPress == true then 
						self.dx	= self.dx - self.moveSpeed
					end
		
					--apply movement results according to deltas
					while self.dx > threshold do
						self.x = self.x + 1
						self.dx = self.dx - threshold
					end
					while self.dx < (-1)*threshold do
						self.x = self.x - 1
						self.dx = self.dx + threshold
					end
		
					--apply bounds
					if self.x > self.boundsRight then
						self.x = self.boundsRight
					end
					if self.x < self.boundsLeft then
						self.x = self.boundsLeft
					end
		
					--# adjust positions of the boundingbox and mousebox
					--##################################################
			
			
					self.adjustFunctionBoxes(self) --does: self.adjustBoundingBox(self) + self.adjustMouseBox(self)
		
		
					--# prepare trigger for shooting
					--####################################
	
					local shootAttempt = false
			
					if self.firePress == true then
		
						--while shooting button pressed count up until trigger reloaded
						self.leadPlBulletProgress = self.leadPlBulletProgress + self.leadPlBulletStep
		
						if self.leadPlBulletProgress > self.leadPlBulletThresh then
							self.leadPlBulletProgress = self.leadPlBulletThresh
						end
						shootAttempt = true
				
						--print("shoot attempt")
					else
						--otherwise when shooting not pressed, shortcut to fully reloaded trigger
						self.leadPlBulletProgress = self.leadPlBulletThresh
					end
		
					--# with shooting trigger set, check shooting condition and shoot when allowed
					--############################################################################
	
					if shootAttempt == true then
						--fire pressed and trigger at maximum is first condition neccessary for shooting
						if self.leadPlBulletProgress == self.leadPlBulletThresh then
							self.shootCondition1=true
					
							--print("condition 1")
						end
		
			 
						--number of leadbullets less than maximum is the second condition neccessary for shooting
						--Step 1: iterate over global bulletcontainer maximum, check how many bullets are in it
						local bCount=0
						for i= 1,globLeadPlBulletIndexMax,1 do
					
							if globLeadPlBullets[self.playerNumColor][i].type ~= PLBULLET_INACTIVE then --TODO check if correct bullet container is accessed
								bCount = bCount + 1
							end
						end
						--Step 2: check if bullet-maximum of player is reached, if there is room for new bullet, set condition 2
						--print("bCount"..bCount)
						if bCount < self.leadPlBulletMax then
							self.shootCondition2=true
					
							--print("condition 2")
						end
		
						if self.shootCondition1 == true and self.shootCondition2 == true then
		
							--print("shoot")
		
							self.fireShot(self)
		
							self.leadPlBulletProgress = 0--reset the trigger
						end
		
						--do not carry shooting conditions over to the next update
						self.shootCondition1=false
						self.shootCondition2=false
					end	
			
			
					--Respawn handling old version (TODO may want to place new version here)
			
					--if self.RespawnCountDownIndex > self.RespawnCountDownResume then
			
					--	self.RespawnCountDownIndex = self.RespawnCountDownIndex -1
				
						--TODO toggle player visibility with RespawnCountDownIndex
				
					--elseif self.RespawnCountDownIndex == self.RespawnCountDownResume then --standard: player starts invulnerable for short period of time
					--	self.respawnedVulnerableFlag = true -- (re-)enable vulnerability after invulnerability countdown
					--	self.RespawnCountDownIndex   = 0
					--end
			
			
		
				--elseif lclSpecialUpdateOnly == true then

				--end
			
			
			
			
			
				end  --act on num of lifes > 0 end
		
			else --active false
			
				self.isVisible = false 
			
			end --act on active end
		
		
		
		elseif lclSpecialUpdateOnly == true then
				
			if self.dyingCountDown > 0 then
				self.dyingCountDown = self.dyingCountDown - 1
						
				if self.dyingCountDown == (self.dyingCountDownMax - 1) then
							
					self.isVisible = false --neccessary but not enough to make invisible or else further playercollisions will be detected as the game runs
					self.active=false --need to put active to false
							
					self.plantPlayerExplosion(self)
					
					if globIsInDemoMode == false then 
						playerExplosionSnd:stop()
						playerExplosionSnd:play()
					end
							
				elseif self.dyingCountDown == 1 then
						
					--finalize dying, do resumechecks, gameoverchecks
					--cannot do the resumechecks here and make player visible again because of the problem that on playergameover
					--the player will be visible for a short period of time before deactiivation. Instead set a resumeCheck flag
					--and have the ship do the resume check outside of the lclSpecialUpdateOnly part
					self.doResumeChecks = true
						
					self.dyingCountDown = 0
				end
			end

		end
		
		
	
	end --act on enteredGame end
	
	
	--There are updates that won't actualize inputs. To avoid carry-over of values to next update, empty them out.
	self.leftPress  = false
	self.rightPress = false
	self.firePress  = false
	
	
end

function PlayerShip.adjustFunctionBoxes(self)
	
	--update any bounding-objects that have a specific function in this player-object
	self.adjustBoundingBoxes(self)
	self.adjustMouseBox(self)
end

function PlayerShip.adjustBoundingBoxes(self)

	--1 frame, 1 box, simplified version
	self.currentBBox.doUpdate(self.currentBBox, self.x, self.y)
	--self.currentBBox[2].doUpdate(self.currentBBox[2], self.x, self.y)--only 1 boundingbox here
end

function PlayerShip.adjustMouseBox(self)

	self.currentMouseBox.doUpdate(self.currentMouseBox, self.x, self.y)
end

function PlayerShip.fireShot(self)
	print("Fire shot was executed!")
	
	--access bullets from globals
	--TODO careful, bullets have no firepower and no animation offset in this game parameter in this game
	
	--SIMPLIFIED
	local playerIdx = self.playerNumColor
	local bulletIdx = globLeadPlBulletIndex[self.playerNumColor] --fetch bulletindex for this players's bulletcontainer
	--with playercolor index
	globLeadPlBullets[playerIdx][bulletIdx].activate(globLeadPlBullets[playerIdx][bulletIdx], self.x + 5, self.y + 1, 0.0, -2.9)
	
	globLeadPlBulletIndex[playerIdx] = globLeadPlBulletIndex[playerIdx] + 1
	if globLeadPlBulletIndex[playerIdx] > globLeadPlBulletIndexMax then
		globLeadPlBulletIndex[playerIdx] = 1
	end
	
	if globIsInDemoMode == false then 
		shipShotSnd:stop()
		shipShotSnd:play()
	end
	
end

function PlayerShip.plantPlayerExplosion(self)

	
	globPlayerExplosions[globPlayerExplosionIndex].defineColorType(globPlayerExplosions[globPlayerExplosionIndex], self.playerNumColor)
	globPlayerExplosions[globPlayerExplosionIndex].activate(globPlayerExplosions[globPlayerExplosionIndex], self.x - 3, self.y - 5)
	globPlayerExplosionIndex = globPlayerExplosionIndex + 1
	if globPlayerExplosionIndex > globPlayerExplosionMax then
		globPlayerExplosionIndex = 1
	end

end


function PlayerShip.putToDeath(self)

	self.dyingCountDown = self.dyingCountDownMax
end


function PlayerShip.draw(self)

	if self.enteredGame and self.isVisible then
	
		if self.currentGph ~= nil then
			lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
		else
			dummy=1/0 --ought not to be nil once drawing is attempted. if so, crash deliberately on this line
		end
		
		--debug
		--self.currentMouseBox.draw(self.currentMouseBox)
	end
end




