--requires
	--gamestate = require "libs/gamestate" --no use of hump
	Object = require "libs.classic"
	
--luas requires
	require "luas.extendedLoader"
	require "luas.gameMode"
	require "luas.demoMode"
	require "luas.globals"
	
--gamestate constants																					--Task AAA Definition of Gamestate Constants
	STATE_UNDEF = (-1)
	STATE_INIT = 0
	STATE_USE_GAME = 1
	STATE_SWITCH_GAME_DEMO = 2 --good idea: use isolated state-variables for gamestate transitions and their cleanups and re-initializations
	STATE_USE_DEMO = 3         --then use state-variables for actual gamestates to indicate they are to be used now
	STATE_SWITCH_DEMO_GAME = 4
	STATE_SWITCH_INIT_GAME = 5 --also include transitions from init-state to either demoMode or gameMode
	STATE_SWITCH_INIT_DEMO = 6
	
--color constants
	P_RED   = 1																							--Task AAB Definition of color constants
	P_GREEN = 2
	P_BLUE  = 3
	P_ALPHA = 4	
	
function love.load()

	screenscale = 3--resize to 3 pixel 																	--Task AAC Declaration of screen resolution values
	
	resWidth = 200-- 600 --resize to 3 pixel 
	resHeight= 150-- 450
	
	gStateIndex    = STATE_INIT																			--Task AAD Declaration of Gamestate variables
	
	gStateDemo = nil
	gStateGame = nil
	
	dUpd = 0.0																							--Task AAE initialisation of game timing variables and threshold
	tUpd = 0.025
	threshold = 1.0
	epsilon   = 0.000001
	
	cnv= love.graphics.newCanvas(resWidth, resHeight)													--Task AAF setting of game canvas
	cnv:setFilter("nearest","nearest")
	
	--TODO load functions from extened loader
	loadLevelContents()																					--Task AAG (part 1) load game graphics
	loadPlayerSprites()
	loadEnemySprites()
	loadLevelGphs()
	loadProjectileSprites()
	loadMiscSprites()
	loadSymbols()
	loadOverlays()
																										--Task AAH (part 1) load sound
	--loadMp3() --not used
	loadSfx()
	
	--debug
	svarDemo=0
	svarGame=0
	
end	

function love.update(dt)																				--Task AAI Game update


	dUpd = dUpd + dt
	
	if dUpd > tUpd then
		dUpd = dUpd - tUpd
		
		doFidelityBlasterUpdate() --if main-file is to control multiple games, put update branches here
		
	end			
	
end


function doFidelityBlasterUpdate()																		--Task AAJ Game statemachine
	
	if gStateIndex == STATE_INIT then																	--Task AAK Init State, initialize gamestates for demomode and gamemode, init globals
	
		gStateDemo = DemoMode()
		gStateGame = GameMode()
		
		globInit() --initialize global values and objects												--Task AAL (part 1) init globals
		
		gStateIndex = STATE_SWITCH_INIT_DEMO --(TODO) later replace with transition to demoMode
	
	elseif gStateIndex == STATE_USE_GAME then
	
		if svarGame<5 then
			print("keep updating gameMode")
			svarGame= svarGame +1
		end
	
		--update gameMode
		gStateGame.doUpdate(gStateGame)																	--Task AAM (part 1) update game mode
	
		if gStateGame.leaveStateReady == true then														--Task AAN (part 1) exit game mode to demo (after gameover)
			--go to gamestate transition for game-to-demo
			gStateIndex = STATE_SWITCH_GAME_DEMO
			
			print("demoMode switch condition")
		end
	
	elseif gStateIndex == STATE_SWITCH_GAME_DEMO or gStateIndex == STATE_SWITCH_INIT_DEMO then			--Task AAO statemachine transitions to demo mode	
	
		--do cleanups, and re-initialisations
		globResetDemo()																					--Task AAP (part 1) call globals reset for demo
		
		--it's best to create a new instance for the gamestate to transition to
		gStateDemo = DemoMode()
		
		--leave for other gamestate
		gStateIndex = STATE_USE_DEMO
		
		print("switch to demoMode")
	
	elseif gStateIndex == STATE_USE_DEMO then
		if svarDemo<5 then
			print("keep updating demoMode")
			svarDemo= svarDemo +1
		end
		
		--update demoMode
		gStateDemo.doUpdate(gStateDemo)																	--Task AAQ (part 1) update demo mode
	
		if gStateDemo.leaveStateReady == true then
			--go to gamestate transition for demo-to-game
			gStateIndex = STATE_SWITCH_DEMO_GAME
			
			print("gameMode switch condition")
		end
	
													  --DO NOT ALLOW THIS TRANSITION:
	elseif gStateIndex == STATE_SWITCH_DEMO_GAME then --or gStateIndex == STATE_SWITCH_INIT_GAME then	--Task AAR statemachine transition to game mode	
	
		--do cleanups, and re-initialisations
		globResetGame()																					--Task AAS (part 1) call globals reset for game mode
		
		--it's best to create a new instance for the gamestate to transition to
		gStateGame = GameMode()
		
		--leave for other gamestate
		gStateIndex = STATE_USE_GAME
		
		print("switch to gameMode")
	
	else
		--undefined case, let the game crash to show there is an error
		local dummy = 1/0
		
	end

end

function love.draw()																					--Task AAT (part 1) draw canvas
	
	love.graphics.setCanvas(cnv)
	love.graphics.clear()
	
	--gStateCurrent.draw(gStateCurrent)
	
	doFidelityBlasterDraw()  --if main-file is to control multiple games, put drawing branches here
	
	
	love.graphics.setCanvas()
	--love.graphics.scale(screenscale, screenscale) --alternative
	
	love.graphics.draw(cnv,0,0,0,screenscale,screenscale)
end

function doFidelityBlasterDraw()																			--Task AAT (part 2) draw canvas demo mode and game mode separately
	if gStateIndex == STATE_USE_DEMO then
		if gStateDemo ~= nil then
			gStateDemo.draw(gStateDemo)
		end
	
	elseif gStateIndex == STATE_USE_GAME then
		if gStateGame ~= nil then
			gStateGame.draw(gStateGame)
		end
	
	end
end


